#!/usr/bin/env python
# coding: utf-8

class ObjectId(object):
    '''
    Class that represents a Object ID.
    '''

    def __init__(self, data={}):
        '''
        Args:
            data (dict): objectId fields
        Dictionary keys:
            'timestamp'
            'counter'
            'machineIdentifier'
            'processIdentifier'
            'timeSecond'
            'time'
            'date'
        '''

        self.timestamp = data.pop('timestamp', None)
        self.counter = data.pop('counter', None)
        self.machineIdentifier = data.pop('machineIdentifier', None)
        self.processIdentifier = data.pop('processIdentifier', None)
        self.timeSecond = data.pop('timeSecond', None)
        self.time = data.pop('time', None)
        self.date = data.pop('date', None)


    def serialize(self, creating=False):
        '''
        Return JSON serializable dictionary.
        '''
        return {
            'timestamp'         : self.timestamp,
            'counter'           : self.counter,
            'machineIdentifier' : self.machineIdentifier,
            'processIdentifier' : self.processIdentifier,
            'timeSecond'        : self.timeSecond,
            'time'              : self.time,
            'date'              : self.date
        }



#!/usr/bin/env python
# coding: utf-8

from trivoreid.models.objectid import ObjectId

class MyDataPackage(object):
    '''
    Class that represents MyData Package.
    '''

    def __init__(self, data):
        '''
        Args:
            data (dict): MyData package fields
        Dictionary keys:
            'id' (str)              : package ID
            'userId' (str)          : user ID
            'status' (MyDataStatus) : status
            'dataEntries' (list)    : list of MyDataEntry objects
            'collectedOn'           : time when the package was collected
            'collectingStartedOn'   : time when collecting started
        '''

        self.id = data.pop('id', None)
        self.userId = data.pop('userId', None)
        self.status = data.pop('status', None)
        self.data_entries = []
        self.collectedOn = data.pop('userId', None)
        self.collectingStartedOn = data.pop('userId', None)

        for entry in data.pop('dataEntries', []):
             self.data_entries.append(MyDataEntry(entry))

    def serialize(self):

        data_entries = []
        for d in self.data_entries:
            data_entries.append(d.serialize())

        return {
            'id'                    : self.id,
            'userId'                : self.userId,
            'status'                : self.status,
            'data_entries'          : data_entries,
            'collectedOn'           : self.collectedOn,
            'collectingStartedOn'   : self.collectingStartedOn
        }

class MyDataEntry(object):
    '''
    Class that represents MyData entry.
    '''

    def __init__(self, data):
        '''
        Args:
            data (dict): MyData entry fields
        Dictionary keys:
            'code' (str)                    : entry code
            'dataId' (str)                  : data ID
            'status' (str)                  : status
            'data' (MyData)                 : MyData object
            'document' (dict)               : document
            'backendSystemName' (str)
            'mimeType' (str)
            'account' (str)
            'updatedOn' (str)
            'dataIdAsObjectId' (ObjectId)
            'backendSystem' (str)
        '''
        self.code = data.pop('code', None)
        self.status = data.pop('status', None)
        self.document = data.pop('document', {})
        self.data = MyData(data.pop('data', {}))
        self.backendSystemName = data.pop('backendSystemName', None)
        self.mimeType = data.pop('mimeType', None)
        self.account = data.pop('account', None)
        self.updatedOn = data.pop('updatedOn', None)
        self.dataIdAsObjectId = ObjectId(data.pop('dataIdAsObjectId', None))
        self.dataId = data.pop('dataId', None)
        self.backendSystem = data.pop('backendSystem', None)

    def serialize(self):
        return {
            'code'              : self.code,
            'dataId'            : self.dataId,
            'status'            : self.status,
            'document'          : self.document,
            'data'              : self.data.serialize(),
            'backendSystemName' : self.backendSystemName,
            'mimeType'          : self.mimeType,
            'account'           : self.account,
            'updatedOn'         : self.updatedOn,
            'dataIdAsObjectId'  : self.dataIdAsObjectId.serialize(),
            'backendSystem'     : self.backendSystem,
        }

class MyData(object):
    '''
    Class that represents MyData object.
    '''

    def __init__(self, data):
        '''
        Args:
            data (dict): MyData fields
        Dictionary keys:
            'id' (str)              : entry ID
            'data' (bytes)          : MyData
        '''
        self.id = data.pop('id', None)
        self.data = data.pop('data', None)

    def serialize(self):
        return {
            'id'   : self.id,
            'data' : self.data
        }



#!/usr/bin/env python
# coding: utf-8

from trivoreid.models.misc import Meta
from trivoreid.models.email import EmailAddress
from trivoreid.exceptions import TrivoreIDSDKException

class User(object):
    '''
    The User class that represents a user object.
    '''

    def __init__(self, data={}):
        '''
        Args:
            data (dict): user fields
        Dictionary keys:
            'id' - user ID
            'username' - sign in name
            'userAccountType' - PERSON or LEGAL_ENTITY
            'nsCode' - namespace code
            'emails' - list of emails. Object of 'EmailAddresses' class
            'mobiles' - list of mobiles. Object of 'Mobiles' class
            'name' - user names. Object of 'Names' class
            'memberOf' - the list of group IDs (when creating new user or
                         modifying existing - can be group names)
            'addresses' - list of addresses. Object of 'Addresses' class

            'locale' - user's preferred locale
            'preferredLanguage' - user's preferred language
            'timeZone' - user's preferred timezone
            'mfaMethod' - multi-factor authentication method (TOTP, SMS or NONE)
            'nickName' - user's nickname or preferred name
            'locked' - user's locked status
            'minor' - is user a legal minor (client defined age). Can be used
                      as alternative to dateOfBirth.
            'websiteMain' - main website
            'websiteAux' - auxiliary website
            'domicileCode' -
            'strongIdentification' - information about last strong
                                     identification event
            'customerGroupInfo' - customer service entered data regarding
                                  customer's special statuses
            'dateOfBirth' - date of birth
            'meta' - meta data

            'gender' - Free-form field for gender
            'sex' - The juridical sex of the user. 'FEMALE', 'MALE', 'OTHER',
                    'UNDISCLOSED'.
            'nationality' - Represents the user's nationality in format
                            ISO 3166-1 Alpha-2, e.g. DE
            'salutation' - End-user’s salutation, e.g. "Mr."
            'title' - End-user’s title, e.g. "Dr."
            'birthFirstName' - First name someone has when they are born, or at
                               least from the time they are a child. This term
                               can be used by a person, who changes the first
                               name later in life for any reason.
            'birthMiddleName' - Middle name someone has when they are born, or
                                at least from the time they are a child. This
                                term can be used by a person, who changes the
                                middle name later in life for any reason.
            'birthLastName' - Last name someone has when he or she is born, or
                              at least from the time he or she is a child. This
                              term can be used by a person who changes the
                              family name later in life for any reason.
            'place_of_birth' - End-user’s place of birth.
        '''

        mobiles = []
        for m in data.pop('mobiles', []):
            mobiles.append(Mobile(m))

        addresses = []
        for a in data.pop('addresses', []):
            addresses.append(Address(a))

        emails = []
        for e in data.pop('emails', []):
            emails.append(EmailAddress(e))

        self.id = data.pop('id', None)
        self.username = data.pop('username', None)
        self.userAccountType = data.pop('userAccountType', None)
        self.nsCode = data.pop('nsCode', None)
        self.memberOf = data.pop('memberOf', [])
        self.name = Names(data.pop('name', {}))
        self.mobiles = mobiles
        self.emails = emails
        self.addresses = addresses
        self.consents = Consents(data.pop('consents', {}))
        self.password = None
        self.locale = data.pop('locale', None)
        self.preferredLanguage = data.pop('preferredLanguage', None)
        self.timeZone = data.pop('timeZone', None)
        self.mfaMethod = data.pop('mfaMethod', None)
        self.nickName = data.pop('nickName', None)
        self.locked = data.pop('locked', False)
        self.minor = data.pop('minor', False)
        self.websiteMain = data.pop('websiteMain', None)
        self.websiteAux = data.pop('websiteAux', None)
        self.domicileCode = data.pop('domicileCode', None)
        self.domicileClasses = data.pop('domicileClasses', [])
        self.dateOfBirth = data.pop('dateOfBirth', None)
        self.strongIdentification = StrongIdentification(
                                           data.pop('strongIdentification', {}))
        self.customerGroupInfo = CustomerGroupInfo(
                                           data.pop('customerGroupInfo', {}))
        self.meta = Meta(data.pop('meta', {}))
        self.gender = data.pop('gender', None)
        self.sex = data.pop('sex', None)
        self.nationality = data.pop('nationality', None)
        self.salutation = data.pop('salutation', None)
        self.title = data.pop('title', None)
        self.birthFirstName = data.pop('birthFirstName', None)
        self.birthMiddleName = data.pop('birthMiddleName', None)
        self.birthLastName = data.pop('birthLastName', None)
        self.placeOfBirth = PlaceOfBirth(data.pop('place_of_birth', {}))
        self.tags = data.pop('tags', [])

    def serialize(self):
        '''
        Return dictionary that is JSON serializable.
        '''

        mobiles = []
        for m in self.mobiles:
            mobiles.append(m.serialize())

        addresses = []
        for a in self.addresses:
            addresses.append(a.serialize())

        emails = []
        for e in self.emails:
            emails.append(e.serialize())

        user_serializable = {
            'id': self.id,
            'username': self.username,
            'userAccountType': self.userAccountType,
            'nsCode': self.nsCode,
            'emails': emails,
            'mobiles': mobiles,
            'addresses': addresses,
            'name':  self.name.serialize(),
            'consents': self.consents.serialize(),
            'memberOf': self.memberOf,
            'locale': self.locale,
            'preferredLanguage': self.preferredLanguage,
            'timeZone': self.timeZone,
            'mfaMethod': self.mfaMethod,
            'nickName': self.nickName,
            'locked': self.locked,
            'minor': self.minor,
            'websiteMain': self.websiteMain,
            'websiteAux': self.websiteAux,
            'domicileCode': self.domicileCode,
            'domicileClasses': self.domicileClasses,
            'dateOfBirth': self.dateOfBirth,
            'customerGroupInfo': self.customerGroupInfo.serialize(),
            'strongIdentification': self.strongIdentification.serialize(),
            'meta': self.meta.serialize(),
            'gender': self.gender,
            'sex': self.sex,
            'nationality': self.nationality,
            'salutation': self.salutation,
            'title': self.title,
            'birthFirstName': self.birthFirstName,
            'birthMiddleName': self.birthMiddleName,
            'birthLastName': self.birthLastName,
            'place_of_birth': self.placeOfBirth.serialize(),
            'tags': self.tags
        }

        if self.password is not None:
            user_serializable['password'] = self.password

        return user_serializable

class Names(object):
    '''
    The object containing user's given, middle and family names.
    '''

    def __init__(self, data={}):
        '''
        Args:
            data (dict): name fields
        Dictionary keys:
            'givenName' - user's given name
            'middleName' - user's middle name
            'familyName' - user's family name
        '''
        self.givenName = data.pop('givenName', None)
        self.middleName = data.pop('middleName', None)
        self.familyName = data.pop('familyName', None)

    def serialize(self):
        '''
        Return dictionary that is JSON serializable.
        '''
        return {
            'givenName'  : self.givenName,
            'middleName' : self.middleName,
            'familyName' : self.familyName,
        }

class Mobile(object):
    '''
    The object containing the mobile data.
    '''
    def __init__(self, data={}):
        '''
        Args:
            data (dict, str) : mobile fields, or single mobile number.
        Dictionary keys:
            'number' (str) - mobile number
            'verified' (bool) - verification status
        '''
        if type(data) is dict:
            self.name = data.pop('name', None)
            self.number = data.pop('number', None)
            self.verified = data.pop('verified', False)
            self.tags = data.pop('tags', [])
        elif type(data) is str:
            self.number = data
            self.name = None
            self.verified = None
            self.tags = None
        else:
            raise TrivoreIDSDKException('Wrong type of the mobile!')

    def serialize(self):
        '''
        Return dictionary that is JSON serializable.
        '''
        return {
            'name'      : self.name,
            'number'    : self.number,
            'verified'  : self.verified,
            'tags'      : self.tags
        }

class Address(object):
    '''
    The object containing the user address.
    '''

    def __init__(self, data={}):
        '''
        Args:
            data (dict): address fields
        Dictionary keys:
            'addressName'
            'name'
            'country'
            'locality'
            'postalCode'
            'region'
            'streetAddress'
            'type' PERMANENT or TEMPORARY
            'organisation'
            'verified'
            'source'
            'language' ISO 639 2-letter language code.
        '''
        self.addressName = data.pop('addressName', None)
        self.name = data.pop('name', None)
        self.country = data.pop('country', None)
        self.locality = data.pop('locality', None)
        self.region = data.pop('region', None)
        self.postalCode = data.pop('postalCode', None)
        self.streetAddress = data.pop('streetAddress', None)
        self.type = data.pop('type', None)
        self.organisation = data.pop('organisation', None)
        self.verified = data.pop('verified', None)
        self.source = data.pop('source', None)
        self.language = data.pop('language', None)

    def serialize(self):
        '''
        Return dictionary that is JSON serializable.
        '''
        return {
            'addressName': self.addressName,
            'name': self.name,
            'country': self.country,
            'locality': self.locality,
            'postalCode': self.postalCode,
            'region': self.region,
            'streetAddress': self.streetAddress,
            'type': self.type,
            'organisation': self.organisation,
            'verified': self.verified,
            'source': self.source,
            'language': self.language
        }

class Consents(object):
    '''
    Wrapper for the user's marketing consents.
    '''

    def __init__(self, data={}):
        '''
        Args:
            data (dict): user's consents
        Dictionary keys:
            'marketingPost' (bool)
            'marketingEmail' (bool)
            'marketingPhone' (bool)
            'marketingMobileMessage' (bool)
            'marketingPushNotification' (bool)
            'marketingOther' (bool)
            'locationing' (bool)
            'profiling' (bool)
        '''
        self.marketingPost = data.pop('marketingPost', False)
        self.marketingEmail = data.pop('marketingEmail', False)
        self.marketingPhone = data.pop('marketingPhone', False)
        self.marketingMobileMessage = data.pop('marketingMobileMessage', False)
        self.marketingPushNotification = data.pop('marketingPushNotification', False)
        self.marketingOther = data.pop('marketingOther', False)
        self.locationing = data.pop('locationing', False)
        self.profiling = data.pop('profiling', False)

    def serialize(self):
        '''
        Return dictionary that is JSON serializable.
        '''
        return {
            'marketingPost': self.marketingPost,
            'marketingEmail': self.marketingEmail,
            'marketingPhone': self.marketingPhone,
            'marketingMobileMessage': self.marketingMobileMessage,
            'marketingPushNotification': self.marketingPushNotification,
            'marketingOther': self.marketingOther,
            'locationing': self.locationing,
            'profiling': self.profiling,
        }

class CustomerGroupInfo(object):
    '''
    Wrapper for the user's customer group info.
    '''

    def __init__(self, data={}):
        '''
        Args:
            data (dict): user's consents
        Dictionary keys:
            'student' (bool)
            'studentIdentifier' (str)
            'studentValidFrom' (str)
            'studentValidTo' (str)
            'studentUpdated' (str)
            'senior' (bool)
            'seniorIdentifier' (str)
            'seniorUpdated' (str)
            'disabilities' (bool)
            'disabilitiesIdentifier' (str)
            'disabilitiesUpdated' (str)
            'studentValidNow' (bool)
        '''

        self.student = data.pop('student', None)
        self.studentIdentifier = data.pop('studentIdentifier', None)
        self.studentValidFrom = data.pop('studentValidFrom', None)
        self.studentValidTo = data.pop('studentValidTo', None)
        self.studentUpdated = data.pop('studentUpdated', None)
        self.senior = data.pop('senior', None)
        self.seniorIdentifier = data.pop('seniorIdentifier', None)
        self.seniorUpdated = data.pop('seniorUpdated', None)
        self.disabilities = data.pop('disabilities', None)
        self.disabilitiesIdentifier = data.pop('disabilitiesIdentifier', None)
        self.disabilitiesUpdated = data.pop('disabilitiesUpdated', None)
        self.studentValidNow = data.pop('studentValidNow', None)

    def serialize(self):
        return {
            'student': self.student,
            'studentIdentifier': self.studentIdentifier,
            'studentValidFrom': self.studentValidFrom,
            'studentValidTo': self.studentValidTo,
            'studentUpdated': self.studentUpdated,
            'senior' : self.senior,
            'seniorUpdated': self.seniorUpdated,
            'seniorIdentifier': self.seniorIdentifier,
            'disabilities': self.disabilities,
            'disabilitiesIdentifier': self.disabilitiesIdentifier,
            'disabilitiesUpdated': self.disabilitiesUpdated,
            'studentValidNow': self.studentValidNow
        }

class StrongIdentification(object):
    '''
    Wrapper for the user's strong identification info.
    '''

    def __init__(self, data={}):
        '''
        Args:
            data (dict): user's strong identification fields
        Dictionary keys:
            'method' (str) method used to strongly identify user. One of:
                           IN_PERSON, SUOMI_FI, SUOMI_FI_VALTUUDET, USER_API
            'identificationDocuments' (list) documents used during
                                             identification.
            'otherExplanation' (str) other explanation related to identification.
            'personalId' (str) user's personal ID. May be censored if viewer
                               doesn't have permission ACCOUNT_VIEW_PERSONAL_ID.
            'remarks' (str) remarks related to last strong identification event.
            'identifier' (str) user name, client ID, or other who performed last
                               identification.
            'time' (str) timestamp when user was strongly identified.
            'firstTime' (str) first time when user was strongly identified.
            'count' (int) number of times user has been strongly identified.
            'identified' (bool) has user been strongly identified.
        '''

        self.method = data.pop('method', None)
        self.identificationDocuments = data.pop('identificationDocuments', None)
        self.otherExplanation = data.pop('otherExplanation', None)
        self.personalId = data.pop('personalId', None)
        self.remarks = data.pop('remarks', None)
        self.identifier = data.pop('identifier', None)
        self.time = data.pop('time', None)
        self.firstTime = data.pop('firstTime', None)
        self.count = data.pop('count', None)
        self.identified = data.pop('identified', None)

    def serialize(self):
        return {
            'method': self.method,
            'identificationDocuments': self.identificationDocuments,
            'otherExplanation': self.otherExplanation,
            'personalId': self.personalId,
            'remarks': self.remarks,
            'identifier': self.identifier,
            'time': self.time,
            'firstTime': self.firstTime,
            'count': self.count,
            'identified': self.identified
        }

class PasswordUpdateResult(object):
    '''
    Wrapper for the user's password update result.
    '''

    def __init__(self, data={}):
        '''
        Args:
            data (dict): user's password update result fields
        Dictionary keys:
            'success' (bool)
            'validationErrors' (list) list of ValidationError objects
        '''

        self.validationErrors = []
        for v in data.pop('validationErrors', []):
            self.validationErrors.append(ValidationError(v))

        self.success = data.pop('success', None)

    def serialize(self):

        validationErrors = []
        for v in self.validationErrors:
            validationErrors.append(v.serialize())

        return {
            'success': self.success,
            'validationErrors' : validationErrors
        }

class PlaceOfBirth(object):
    '''
    Wrapper for the user’s place of birth.
    '''

    def __init__(self, data={}):
        '''
        Args:
            data (dict): user’s place of birth fields
        Dictionary keys:
            'country'
            'locality'
            'region'
        '''
        self.country = data.pop('country', None)
        self.locality = data.pop('locality', None)
        self.region = data.pop('region', None)

    def serialize(self):
        return {
            'country': self.country,
            'locality': self.locality,
            'region': self.region
        }

class ValidationError(object):
    '''
    Wrapper for the validation error.
    '''

    def __init__(self, data={}):
        '''
        Args:
            data (dict): validation error
        Dictionary keys:
            'message' (str)
            'code' (str) validation error code
        '''
        self.message = data.pop('message', None)
        self.code = data.pop('code', None)

    def serialize(self):
        return {
            'message': self.message,
            'code' : self.code
        }

class NamespaceMigrationOptions(object):
    '''
    Wrapper for user namespase migrate options.
    '''

    def __init__(self, data={}):
        '''
        Args:
            data (dict): namespace migrate options
        Dictionary keys:
            'targetNsCode' (str) Target namespace's code.
            'keepUserId' (bool) default: True. If true, same user ID will be
                                kept. If false, a new user ID is given. Some
                                internal references to original ID may be
                                deleted during migration even if the same ID
                                is kept.
            'keepProfile' (bool) default: True. If true: names, street
                                 addresses, legalinfo, customergroupinfo, DoB,
                                 and other profile data will be migrated.
            'keepEmails' (bool) default: True. If true, email addresses and
                                verification status will be migrated.
            'keepMobiles' (bool) default: True. If true, mobile numbers and
                                 verification status will be migrated.
            'keepConsents' (bool) default: True. If true, user consents will
                                  be migrated.
            'keepCustomFields' (bool) default: True. If true, custom fields
                                      will be migrated.
            'keepPassword' (bool) default: True. If true: password will be
                                  migrated unless new password is specified.
            'keepGroups' (bool) default: False. If true: user groups will be
                                migrated based on group names.
            'username' (str) Username after migration. If null, username is
                             autogenerated. Value must be unique in new
                             namespace.
            'userPassword' (str) Password for new migrated user.
        '''
        self.targetNsCode = data.pop('targetNsCode', None)
        self.keepUserId = data.pop('keepUserId', True)
        self.keepProfile = data.pop('keepProfile', True)
        self.keepEmails = data.pop('keepEmails', True)
        self.keepMobiles = data.pop('keepMobiles', True)
        self.keepConsents = data.pop('keepConsents', True)
        self.keepCustomFields = data.pop('keepCustomFields', True)
        self.keepPassword = data.pop('keepPassword', True)
        self.keepGroups = data.pop('keepGroups', False)
        self.username = data.pop('username', None)
        self.userPassword = data.pop('userPassword', None)

    def serialize(self):
        return {
            'targetNsCode': self.targetNsCode,
            'keepUserId' : self.keepUserId,
            'keepProfile' : self.keepProfile,
            'keepEmails' : self.keepEmails,
            'keepMobiles' : self.keepMobiles,
            'keepConsents' : self.keepConsents,
            'keepCustomFields' : self.keepCustomFields,
            'keepPassword' : self.keepPassword,
            'keepGroups' : self.keepGroups,
            'username' : self.username,
            'userPassword' : self.userPassword
        }

class LegalInfo(object):
    '''
    The LetalInfo class that represents user's legal information.
    '''

    def __init__(self, data={}):
        '''
        Args:
            data (dict): user legal info fields
        Dictionary keys:
            'firstNames' (str)
            'lastName' (str)
            'callingName' (str)
            'email' (str)
            'phone' (str)
            'domicile' (LegalDomicile)
            'personalIdentityCode' (str)
            'protectionOrder' (boolean)
            'dateOfBirth' (str)
            'dateOfDeath' (str)
            'addresses' (list of trivoreid.models.user.Address)
            'lastUpdatedAt' (str)
            'studentInfo' (StudentInfo)
        '''

        addresses = []
        for a in data.pop('addresses', []):
            addresses.append(Address(a))

        self.firstNames = data.pop('firstNames', None)
        self.lastName = data.pop('lastName', None)
        self.callingName = data.pop('callingName', None)
        self.email = data.pop('email', None)
        self.phone = data.pop('phone', None)
        self.domicile = LegalDomicile(data.pop('domicile', {}))
        self.personalIdentityCode = data.pop('personalIdentityCode', None)
        self.protectionOrder = data.pop('protectionOrder', None)
        self.dateOfBirth = data.pop('dateOfBirth', None)
        self.dateOfDeath = data.pop('dateOfDeath', None)
        self.addresses = addresses
        self.lastUpdatedAt = data.pop('lastUpdatedAt', None)
        self.studentInfo = 	StudentInfo(data.pop('studentInfo', {}))

    def serialize(self):
        addresses = []
        for a in self.addresses:
            addresses.append(a.serialize())

        return {
            'firstNames': self.firstNames,
            'lastName': self.lastName,
            'callingName': self.callingName,
            'email': self.email,
            'phone': self.phone,
            'domicile': self.domicile.serialize(),
            'addresses': addresses,
            'studentInfo': self.studentInfo.serialize(),
            'personalIdentityCode': self.personalIdentityCode,
            'protectionOrder': self.protectionOrder,
            'dateOfBirth': self.dateOfBirth,
            'dateOfDeath': self.dateOfDeath,
            'lastUpdatedAt': self.lastUpdatedAt
        }

class LegalDomicile(object):
    '''
    LegalDomicile class that represents user's legal domicile object.
    '''
    def __init__(self, data={}):
        '''
        Args:
            data (dict): user legal domicile fields
        Dictionary keys:
            'names' (dict)
            'code' (str)
        '''

        self.names = data.pop('names', {})
        self.code = data.pop('code', None)

    def serialize(self):
        return {
            'names': self.names,
            'code': self.code
        }

class StudentInfo(object):
    '''
    StudentInfo class that represents user's student info.
    '''
    def __init__(self, data={}):
        '''
        Args:
            data (dict): user student state fields
        Dictionary keys:
            'state' (StudentStatus)
            'studentValidFrom' (str)
            'studentValidTo' (str)
            'informationUpdated' (str)
            'remoteState' (str)
            'notStudentReason' (str)
            'queryConsentEnds' (str)
        '''

        self.state = data.pop('state', None)
        self.studentValidFrom = data.pop('studentValidFrom', None)
        self.studentValidTo = data.pop('studentValidTo', None)
        self.informationUpdated = data.pop('informationUpdated', None)
        self.remoteState = data.pop('remoteState', None)
        self.notStudentReason = data.pop('notStudentReason', None)
        self.queryConsentEnds = data.pop('queryConsentEnds', None)

    def serialize(self):
        return {
            'state': self.state,
            'studentValidFrom': self.studentValidFrom,
            'studentValidTo': self.studentValidTo,
            'informationUpdated': self.informationUpdated,
            'remoteState': self.remoteState,
            'notStudentReason': self.notStudentReason,
            'queryConsentEnds': self.queryConsentEnds
        }

class StudentState(object):
    '''
    StudentState class that represents user's student info.
    '''
    def __init__(self, data={}):
        '''
        Args:
            data (dict): user student state fields
        Dictionary keys:
            'state' (StudentStatus)
            'studentFrom' (str) Date when user becomes student
            'studentTo' (str) Date when user stops being a student.
            'updated' (str) Timestamp when student status was last updated.
            'lastQuerySuccess' (bool) True if last query from original source
                                      was successful. If false, the student
                                      information was not updated and may be
                                      stale. Client may use the previously known
                                      user information if it is not too old, or
                                      show an error message to the user if
                                      necessary, and try again later.
            'lastQueryError' (str) If lastQuerySuccess was false, this string
                                   contains the known error message. It may be
                                   useful for debugging.
            'source' (str) Database location where state was read from.
        '''

        self.state = data.pop('state', None)
        self.studentFrom = data.pop('studentFrom', None)
        self.studentTo = data.pop('studentTo', None)
        self.updated = data.pop('updated', None)
        self.lastQuerySuccess = data.pop('lastQuerySuccess', None)
        self.lastQueryError = data.pop('lastQueryError', None)
        self.source = data.pop('source', None)

    def serialize(self):
        return {
            'state': self.state,
            'studentFrom': self.studentFrom,
            'studentTo': self.studentTo,
            'updated': self.updated,
            'lastQuerySuccess': self.lastQuerySuccess,
            'lastQueryError': self.lastQueryError,
            'source': self.source
        }

class StudentStatus(object):
    '''
    Student statuses.
    '''
    FULL_TIME = 'FULL_TIME'
    PART_TIME = 'PART_TIME'
    NOT_STUDENT = 'NOT_STUDENT'
    FORBIDDEN = 'FORBIDDEN'
    UNKNOWN = 'UNKNOWN'



#!/usr/bin/env python
# coding: utf-8

from trivoreid.exceptions import TrivoreIDSDKException

class Email(object):
    '''
    Wrapper for the Email message.
    '''
    def __init__(self, data={}):
        '''
        Args:
            data (dict) : email fields
        Dictionary keys:
            'to'            : list of To addresses
            'cc'            : list of CC addresses
            'bcc'           : list of BCC addresses
            'from'          : from email address (string)
            'replyTo'       : list of Reply-To addresses
            'subject'       : email subject
            'html'          : HTML content of email
            'text'          : text content of email
            'attachments'   : attachments to email. In the response this
                              field is omitted.
        '''
        to = []
        for e in data.pop('to', []):
            to.append(EmailAddress(e))

        cc = []
        for e in data.pop('cc', []):
            cc.append(EmailAddress(e))

        bcc = []
        for e in data.pop('bcc', []):
            bcc.append(EmailAddress(e))

        replyTo = []
        for e in data.pop('replyTo', []):
            replyTo.append(EmailAddress(e))

        self.to = to
        self.cc = cc
        self.bcc = bcc
        self.replyTo = replyTo
        self.from_email = data.pop('from', None)
        self.subject = data.pop('subject', None)
        self.html = data.pop('html', None)
        self.text = data.pop('text', None)
        self.attachments = EmailAttachments(data.pop('attachments', {}))

    def construct_email(self,
                        to=None,
                        cc=None,
                        bcc=None,
                        from_email=None,
                        reply_to=None,
                        subject=None,
                        html=None,
                        text=None,
                        attachments=None):
        '''
        Construct email fields.
        NB! Fields will be overwriten!
        Args:
            'to' (list)                 : list of To addresses
            'cc' (list)                 : list of CC addresses
            'bcc' (list)                : list of BCC addresses
            'from_email' (str, dict)    : a single from email address or a
                                          dictionary with email fields.
            'replyTo' (list)            : list of Reply-To addresses
            'subject' (str)             : email subject
            'html' (str)                : HTML content of email
            'text' (str)                : text content of email
            'attachments'               : attachments to email. In the response this
                                          field is omitted.
        '''
        to = self._check_addresses(to)
        cc = self._check_addresses(cc)
        bcc = self._check_addresses(bcc)
        reply_to = self._check_addresses(reply_to)

        if from_email is not None:
            if type(from_email) in [str, dict]:
                from_email = EmailAddress(from_email)
            elif 'EmailAddress' is not str(type(from_email)):
                raise TrivoreIDSDKException('Wrong type of the email!')

        self.to = to
        self.cc = cc
        self.bcc = bcc
        self.replyTo = reply_to
        self.from_email = from_email
        self.subject = subject
        self.html = html
        self.text = text
        self.attachments = attachments

    def serialize(self):
        '''
        Make object JSON serializable.
        '''
        self._check_nulls()

        to = []
        for e in self.to:
            to.append(e.serialize())

        cc = []
        for e in self.cc:
            cc.append(e.serialize())

        bcc = []
        for e in self.bcc:
            bcc.append(e.serialize())

        replyTo = []
        for e in self.replyTo:
            replyTo.append(e.serialize())

        return {
            'to'            : to,
            'cc'            : cc,
            'bcc'           : bcc,
            'from'          : self.from_email.serialize(),
            'replyTo'       : replyTo,
            'subject'       : self.subject,
            'html'          : self.html,
            'text'          : self.text,
            'attachments'   : self.attachments.serialize()
        }

    def _check_addresses(self, data):
        '''
        Check single adddress.
        '''
        if (data == None):
            return []

        if type(data) == list:
            addresses = []
            for e in data:
                addresses.append(EmailAddress(e))
            return addresses
        elif type(data) == str:
            return [EmailAddress(data)]
        elif 'EmailAddress' in str(type(data)):
            return data
        else:
            raise TrivoreIDSDKException('Wrong type of the email!')

    def _check_nulls(self):
        if self.to is None:
            self.to = []
        if self.cc is None:
            self.cc = []
        if self.bcc is None:
            self.bcc = []
        if self.replyTo is None:
            self.replyTo = []
        if self.attachments is None:
            self.attachments = EmailAttachments()
        if self.from_email is None:
            self.from_email = EmailAddress()

class EmailAddress(object):
    '''
    Wrapper for a single email address.
    '''
    def __init__(self, data={}):
        '''
        Args:
            data (dict, str): email fields or a single email address
        Dictionary keys:
            'name'          : email address name
            'address'       : email address
            'verified'      : verification status
            'verifiedBy'    : verified by
            'tags'          : email address tags
        '''
        if type(data) is str:
            self.address = data
            self.name = None
            self.verified = None
            self.verifiedBy = None
            self.tags = None
            self.verifiedDateTime = None
        elif type(data) is dict:
            self.address = data.pop('address', None)
            self.name = data.pop('name', None)
            self.verified = data.pop('verified', False)
            self.verifiedBy = data.pop('verifiedBy', None)
            self.verifiedDateTime = data.pop('verifiedDateTime', None)
            self.tags = data.pop('tags', None)
        else:
            raise TrivoreIDSDKException('Wrong type of the email!')

    def serialize(self):
        '''
        Return dictionary that is JSON serializable.
        '''
        return {
            'name'              : self.name,
            'address'           : self.address,
            'verified'          : self.verified,
            'verifiedBy'        : self.verifiedBy,
            'verifiedDateTime'  : self.verifiedDateTime,
            'tags'              : self.tags
        }

class EmailAttachments(object):
    '''
    Wrapper for the email attachments.
    '''
    def __init__(self, data={}):
        '''
        Args:
            data (dict): attachment fields
        Dictionary keys:
            'name' : attachment file name
            'data' : attachment data, base64 encoded
        '''
        self.attachments = []
        if type(data) is dict and bool(data):
            data = [data]

        for i in range(len(data)):
            self.attachments.append({'name': data[i].pop('name', None),
                                     'data': data[i].pop('data', None)})

    def add_attachment(self, name, data):
        '''
        Args:
            'name' : attachment file name
            'data' : attachment data, base64 encoded
        '''
        self.attachments.append({'name' : name, 'data' : data})

    def serialize(self):
        '''
        Return dictionary that is JSON serializable.
        '''
        return self.attachments



#!/usr/bin/env python
# coding: utf-8

import requests
import string
import random
import logging

import trivoreid.utils.service_utils as su
from trivoreid.utils.criteria import Filter
from trivoreid.models.page import Page
from trivoreid.models.email_template import EmailTemplate, SendMessageOptions
from trivoreid.exceptions import TrivoreIDSDKException, TrivoreIDException


class EmailTemplatesService(object):
    '''
    Class to wrap Trivore ID /emailtemplate APIs
    '''

    _EMAIL_TEMPLATES = 'api/rest/v1/emailtemplate'
    _EMAIL_TEMPLATE = 'api/rest/v1/emailtemplate/{}'
    _EMAIL_TEMPLATE_SEND = 'api/rest/v1/emailtemplate/{}/send'

    def __init__(self, credentials):
        self._server = credentials.server

        if credentials.oidc_client is None:
            self._session = requests
            self._auth = credentials.auth
        else:
            self._session = credentials.oidc_client
            self._auth = None

        self._header = credentials.access_token

    def get_all(self,
                filter_fields=None,
                start_index=0,
                count=100,
                sortBy=None,
                ascending=None):
        '''Get the list of the email templates.
        Args:
            filter_fields (Filter) : filter out the result.
            start_index (int)      : 0-based pagination start index
            count (int)            : pagination page size, max 500,
                                     100 by default
            sortBy (str)           : Sort by attribute name
            ascending (bool)       : Sort direction (ascending or descending)
        Returns:
            Page with the pagination data and the list of email templates.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        params = su.generate_parameters(filter_fields, start_index, count)

        if sortBy != None:
            params['sortBy'] = sortBy

        if ascending != None:
            if ascending:
                params['ascending'] = 'ascending'
            else:
                params['ascending'] = 'descending'

        uri = su.uri(self._server, self._EMAIL_TEMPLATES)
        response = self._session.get(uri, params=params,
                                          headers=self._header,
                                          auth=self._auth)

        if response.status_code is 200:
            logging.debug('Found {} email templates'.format(
                                            len(response.json()['resources'])))
            templates = []
            for template in response.json()['resources']:
                templates.append(EmailTemplate(template))
            page = Page(response.json(), templates)
            return page
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def send(self, emailTemplateId, messageOptions):
        '''Get a single email template by id.
        Args:
            emailTemplateId (str) : unique identifier of the email template that
                                    will be used
            messageOptions (SendMessageOptions) : email template message options
        Raises:
            TrivoreIDException if the status code is not 200.
            TrivoreIDSDKException if the type of the SendMessageOptions is wrong
        '''
        if 'SendMessageOptions' not in str(type(messageOptions)):
            raise TrivoreIDSDKException('SendMessageOptions type is wrong!')

        uri = su.uri(self._server, self._EMAIL_TEMPLATE_SEND
                                            .format(emailTemplateId))
        response = self._session.post(uri,
                                      json=messageOptions.serialize(),
                                      headers=self._header,
                                      auth=self._auth)

        if response.status_code is 204:
            logging.debug('Succesfully sended email with the template with id {}'
                                        .format(emailTemplateId))
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def get(self, emailTemplateId):
        '''Get a single email template details by id.
        Args:
            emailTemplateId (str) : email template's unique identifier
        Returns:
            An email template.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        uri = su.uri(self._server, self._EMAIL_TEMPLATE
                                        .format(emailTemplateId))
        response = self._session.get(uri, headers=self._header,
                                          auth=self._auth)

        if response.status_code is 200:
            logging.debug('Found email template with id {}'
                                        .format(emailTemplateId))
            return EmailTemplate(response.json())
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)



#!/usr/bin/env python
# coding: utf-8

class TrivoreIDSDKException(Exception):
    '''
    An Exception raised when TrivoreID SDK is not properly used.
    '''

    @property
    def message(self):
        return self.__dict__.get('message', None) or getattr(self, 'args')[0]

class TrivoreIDException(Exception):
    '''
    An Exception occured in TrivoreID.
    Args:
        message (str) : error message
        code : server response code
    '''

    def __init__(self, message, code = None):
        self.resporse_code = code
        super().__init__(message)



#!/usr/bin/env python
# coding: utf-8

from trivoreid.models.misc import Meta

class Enterprise(object):
    '''
    Class that represents an enterprise object.
    '''

    def __init__(self, data={}):
        '''
        Args:
            data (dict): enterprise fields
        Dictionary keys:
            'businessId' (str) - business ID
            'vatId' (str)
            'tradeName' (str)
            'domicile' (str)
            'leiCode' (str)
            'duns' (str)
            'parallelTradeNames' (list)
            'auxiliaryTradeNames' (list)
            'tradeRegisterNumber' (str)
            'taxIdentificationNumber' (str)
            'general' (dict) ContactInfo fields
            'sales' (dict) ContactInfo fields
            'service' (dict) ContactInfo fields
            'customerSupport' (dict) ContactInfo fields
            'meta' (dict) meta data
        '''
        self.businessId = data.pop('businessId', None)
        self.vatId = data.pop('vatId', None)
        self.tradeName = data.pop('tradeName', None)
        self.domicile = data.pop('domicile', None)
        self.leiCode = data.pop('leiCode', None)
        self.parallelTradeNames = data.pop('parallelTradeNames', None)
        self.auxiliaryTradeNames = data.pop('auxiliaryTradeNames', None)
        self.tradeRegisterNumber = data.pop('tradeRegisterNumber', None)
        self.taxIdentificationNumber = data.pop('taxIdentificationNumber', None)
        self.general = EnterpriseContactInfo(data.pop('general', {}))
        self.sales = EnterpriseContactInfo(data.pop('sales', {}))
        self.service = EnterpriseContactInfo(data.pop('service', {}))
        self.customerSupport = EnterpriseContactInfo(data.pop('customerSupport', {}))
        self.meta = Meta(data.pop('meta', {}))

    def serialize(self):
        '''
        Return JSON serializable object.
        '''
        return {
            'businessId'                : self.businessId,
            'vatId'                     : self.vatId,
            'tradeName'                 : self.tradeName,
            'domicile'                  : self.domicile,
            'leiCode'                   : self.leiCode,
            'parallelTradeNames'        : self.parallelTradeNames,
            'auxiliaryTradeNames'       : self.auxiliaryTradeNames,
            'tradeRegisterNumber'       : self.tradeRegisterNumber,
            'taxIdentificationNumber'   : self.taxIdentificationNumber,
            'general'                   : self.general.serialize(),
            'sales'                     : self.sales.serialize(),
            'service'                   : self.service.serialize(),
            'customerSupport'           : self.customerSupport.serialize(),
            'meta'                      : self.meta.serialize()
        }

class EnterpriseContactInfo(object):
    '''
    Class that represents a customer support contact info object.
    '''
    def __init__(self, data={}):
        '''
        Args:
            data (dict): contact info fields
        Dictionary keys:
            'phone' (str) - business ID
            'email' (str)
            'website' (str)
            'contacts' (str)
        '''
        self.phone = data.pop('phone', None)
        self.email = data.pop('email', None)
        self.website = data.pop('website', None)
        self.contacts = data.pop('contacts', None)

    def serialize(self):
        '''
        Return JSON serializable object.
        '''
        return {
            'phone'       : self.phone,
            'email'       : self.email,
            'website'     : self.website,
            'contacts'    : self.contacts
        }



#!/usr/bin/env python
# coding: utf-8

class PasswordRequirements(object):
    '''
    The class that represents password requirements.
    '''

    def __init__(self, data = None):
        '''
        Args:
            data (dict) : password requirements
        Dictionary parameters:
            minLength (int) : minimum password length.
            maxLength (int) : maximum password length.
            numberOfCharacteristics (int)
            alphabeticalSequenceMaxLength (int)
            numericalSequenceMaxLength (int)
            qwertySequenceMaxLength (int)
            historyLength (int)
            repeatCharacterLength (int)
            usernameForbidden (bool)
            dictionaryCheck (bool)
        '''
        self.minLength = data.pop('minLength', None)
        self.maxLength = data.pop('maxLength', None)

        self.numberOfCharacteristics = data.pop('numberOfCharacteristics', None)
        self.alphabeticalSequenceMaxLength = data.pop(
                            'alphabeticalSequenceMaxLength', None)
        self.numericalSequenceMaxLength = data.pop(
                            'numericalSequenceMaxLength', None)
        self.qwertySequenceMaxLength = data.pop('qwertySequenceMaxLength', None)
        self.historyLength = data.pop('historyLength', None)
        self.repeatCharacterLength = data.pop('repeatCharacterLength', None)
        self.usernameForbidden = data.pop('usernameForbidden', None)
        self.dictionaryCheck = data.pop('dictionaryCheck', None)

    def serialize(self):
        return {
            'minLength' : self.minLength,
            'maxLength' : self.minLength,
            'numberOfCharacteristics' : self.numberOfCharacteristics,
            'alphabeticalSequenceMaxLength' : self.alphabeticalSequenceMaxLength,
            'numericalSequenceMaxLength' : self.numericalSequenceMaxLength,
            'qwertySequenceMaxLength' : self.qwertySequenceMaxLength,
            'historyLength' : self.historyLength,
            'repeatCharacterLength' : self.repeatCharacterLength,
            'usernameForbidden' : self.usernameForbidden,
            'dictionaryCheck' : self.dictionaryCheck
        }



#!/usr/bin/env python
# coding: utf-8

class EmailTemplate(object):
    '''
    Wrapper for a single email template.
    '''
    def __init__(self, data={}):
        '''
        Args:
            data (dict): email template fields
        Dictionary keys:
            'id' entity ID
            'name' name of email template
            'description' description of email template purpose
            'templateEngine' template engine used to process templates
            'defaultLocale' default locale, used to pick translation if one
                            cannot be picked otherwise
            'defaultTimeZone' default timezone, used if preferred timezone
                              not specified
            'subjectTemplate' template for message's subject field
            'htmlTemplate' template for message's HTML content
            'textTemplate' template for message's plain text content
            'localeProperties' locale specific properties. Those for default
                               locale and those for preferred locale will be
                               used when processing message.
            'namespaceIds' IDs of namespaces where admins and clients can see
                           and use this template
        '''
        self.id = data.pop('id', None)
        self.name = data.pop('name', None)
        self.description = data.pop('description', None)
        self.templateEngine = data.pop('templateEngine', None)
        self.defaultLocale = data.pop('defaultLocale', None)
        self.defaultTimeZone = data.pop('defaultTimeZone', None)
        self.subjectTemplate = data.pop('subjectTemplate', None)
        self.htmlTemplate = data.pop('htmlTemplate', None)
        self.textTemplate = data.pop('textTemplate', None)
        self.localeProperties = data.pop('localeProperties', {})
        self.namespaceIds = data.pop('namespaceIds', [])

    def serialize(self):
        '''
        Return dictionary that is JSON serializable.
        '''
        return {
            'id'                : self.id,
            'name'              : self.name,
            'description'       : self.description,
            'templateEngine'    : self.templateEngine,
            'defaultLocale'     : self.defaultLocale,
            'defaultTimeZone'   : self.defaultTimeZone,
            'subjectTemplate'   : self.subjectTemplate,
            'htmlTemplate'      : self.htmlTemplate,
            'textTemplate'      : self.textTemplate,
            'localeProperties'  : self.localeProperties,
            'namespaceIds'      : self.namespaceIds
        }

class SendMessageOptions(object):
    '''
    Wrapper for sending message options with email templates.
    '''
    def __init__(self, data={}):
        '''
        Args:
            data (dict): email template fields
        Dictionary keys:
            'recipients' message recipients. List of RecipientDetails objects
            'preferredLocale' preferred locale if not specified for recipient
            'preferredTimezone' preferred timezone if not specified
                                for recipient
            'properties' extra properties for template. Recipient properties are
                         added to these. Don't use period characters in keys.
        '''

        self.recipients = []
        for r in data.pop('recipients', []):
            self.recipients.append(RecipientDetails(r))

        self.preferredLocale = data.pop('preferredLocale', None)
        self.preferredTimezone = data.pop('preferredTimezone', None)
        self.properties = data.pop('properties', {})

    def serialize(self):
        '''
        Return dictionary that is JSON serializable.
        '''
        recipients = []
        for r in self.recipients:
            recipients.append(r.serialize())

        return {
            'recipients'        : recipients,
            'preferredLocale'   : self.preferredLocale,
            'preferredTimezone' : self.preferredTimezone,
            'properties'        : self.properties
        }

class RecipientDetails(object):
    '''
    Wrapper for a single recipient details.
    '''
    def __init__(self, data={}):
        '''
        Args:
            data (dict): email template fields
        Dictionary keys:
            'emailAddress' email address that is added to 'To' field. If not
                           specified, user's primary email address is used.
            'userId' recipient user ID. User information can be used
                     in template.
            'preferredLocale' preferred locale, used for translation and
                              formatting. If not specified, User's locale
                              preference is used.
            'preferredTimezone' preferred timezone, used for time and date
                                operations. If not specified, User's timezone
                                preference is used.
            'properties' properties for template. These are added on top of base
                         properties. Don't use period characters in keys.
        '''
        self.emailAddress = data.pop('emailAddress', None)
        self.userId = data.pop('userId', None)
        self.preferredLocale = data.pop('preferredLocale', None)
        self.preferredTimezone = data.pop('preferredTimezone', None)
        self.properties = data.pop('properties', {})

    def serialize(self):
        '''
        Return dictionary that is JSON serializable.
        '''
        return {
            'emailAddress'      : self.emailAddress,
            'userId'            : self.userId,
            'preferredLocale'   : self.preferredLocale,
            'preferredTimezone' : self.preferredTimezone,
            'properties'        : self.properties
        }



#!/usr/bin/env python
# coding: utf-8

import requests
import logging

import trivoreid.utils.service_utils as su
from trivoreid.utils.criteria import Filter
from trivoreid.models.groups import Group
from trivoreid.models.page import Page
from trivoreid.exceptions import TrivoreIDSDKException, TrivoreIDException

class GroupService(object):
    '''
    Class to wrap Trivore ID /group API
    '''

    _GROUPS = 'api/rest/v1/group'
    _GROUP = 'api/rest/v1/group/{}'
    _GROUPS_NESTED = 'api/rest/v1/group/{}/nested'

    def __init__(self, credentials):
        self._server = credentials.server

        if credentials.oidc_client is None:
            self._session = requests
            self._auth = credentials.auth
        else:
            self._session = credentials.oidc_client
            self._auth = None

        self._header = credentials.access_token

    def get_all(self,
                filter_fields=None,
                start_index=0,
                count=100):
        '''Get the list of the groups.
        Args:
            filter_fields (Filter) : utils.Filter class to filter out
                                     the result.
            start_index (int)      : 0-based pagination start index
            count (int)            : pagination page size, max 500,
                                     100 by default
        Returns:
            Page with the pagination data and the list of groups.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        params = su.generate_parameters(filter_fields, start_index, count)
        response = self._session.get(su.uri(self._server, self._GROUPS),
                                    params=params,
                                    headers=self._header,
                                    auth=self._auth)

        if response.status_code == 200:
            logging.debug('Found {} groups'.format(
                        len(response.json()['resources'])))

            groups = []
            for g in response.json()['resources']:
                groups.append(Group(g))
            page = Page(response.json(), groups)
            return page
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def create(self, group):
        '''Create new group.
        Args:
            group (Group) : group to create.
        Returns:
            New group object.
        Raises:
            TrivoreIDSDKException if the type of the Group is wrong
            TrivoreIDException if the status code is not 201.
        '''
        if 'Group' not in str(type(group)):
            raise TrivoreIDSDKException('Group type is wrong!')

        uri = su.uri(self._server, self._GROUPS)
        response = self._session.post(uri,
                                    json=group.serialize(),
                                    headers=self._header,
                                    auth=self._auth)

        if response.status_code == 201:
            group = Group(response.json())
            logging.debug('Successfully created group with id {}'.format(
                                group.id))
            return group
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def get(self, groupId):
        '''Get the single group by id.
        Args:
            groupId (str) : group unique identifier
        Returns:
            A group.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        uri = su.uri(self._server, self._GROUP).format(groupId)
        response = self._session.get(uri, headers=self._header, auth=self._auth)

        if response.status_code is 200:
            logging.debug('Found group with id {}'.format(groupId))
            return Group(response.json())
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def update(self, group, groupId=None):
        '''Modify a group.
        Args:
            group (Group) : group to modify.
            groupId (str) : if None, then group ID from the Group object will
                            be used.
        Returns:
            Modified group object.
        Raises:
            TrivoreIDSDKException if the type of the Group is wrong
            TrivoreIDException if the status code is not 200.
        '''
        if 'Group' not in str(type(group)):
            raise TrivoreIDSDKException('Group type is wrong!')

        if groupId is None:
            groupId = group.id

        uri = su.uri(self._server, self._GROUP).format(groupId)
        response = self._session.put(uri,
                        json = group.serialize(),
                        headers=self._header,
                        auth=self._auth)

        if response.status_code is 200:
            gr = Group(response.json())
            logging.debug('Successfully modified group with the id {}'
                        .format(gr.id))
            return gr
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def delete(self, groupId):
        '''Delete group by id.
        Args:
            groupId (str) : group unique identifier
        Raises:
            TrivoreIDException if the status code is not 204.
        '''
        uri = su.uri(self._server, self._GROUP).format(groupId)
        response = self._session.delete(uri, headers=self._header,
                                             auth=self._auth)

        if response.status_code is 204:
            logging.debug('Successfully deleted group with id {}'.format(groupId))
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def get_nested_groups(self, groupId):
        '''Get all subgroups of the target group.
        Args:
            groupId (str) : target group unique identifier
        Returns:
            A list of subgroups.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        uri = su.uri(self._server, self._GROUPS_NESTED).format(groupId)
        response = self._session.get(uri, headers=self._header, auth=self._auth)

        if response.status_code is 200:
            subgroups = []
            for g in response.json():
                subgroups.append(Group(g))
            logging.debug('Found subgroups for the group {}'.format(groupId))
            return subgroups
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)



#!/usr/bin/env python
# coding: utf-8

def uri(server, api):
    return server + api

def error_response_message(response):
    try:
        errorMessage = response.json()['errorMessage']
    except:
        try:
            errorMessage = ('Unknown error message. Content: {}'
                                                .format(response.content))
        except:
            errorMessage = ('Unknown error message. Content: {}'
                                                .format(response))

    return 'status code {}, {}'.format(response.status_code, errorMessage)

def generate_parameters(filter_fields=None, start_index=0, count=100):
    params = {}

    if (filter_fields != None):
        params['filter'] = filter_fields.generate()

    if (start_index != None):
        params['startIndex'] = start_index

    if (count != None):
        params['count'] = count

    return params

def get_properties(path):
    '''
    Get properties from the property file.
    Returns:
        Dictionary with properties.
    '''
    with open(path) as file:
        lines = [line.split("=") for line in file.readlines()]

    properties = {}
    for l in lines:
        if len(l) == 2: # ignoring comments and empty lines
            properties[l[0]] = l[1].replace('\n', '')

    return properties



#!/usr/bin/env python
# coding: utf-8

__licence__           = 'BSD-3'
__description__       = 'A Python wrapper around Trivore ID API'

from . import exceptions
from . import models
from . import utils
from . import services



#!/usr/bin/env python
# coding: utf-8

import requests
import json
import logging

import trivoreid.utils.service_utils as su
from trivoreid.models.datastorage import DataStorage
from trivoreid.models.page import Page
from trivoreid.exceptions import TrivoreIDSDKException, TrivoreIDException

class DataStorageService(object):
    '''
    Class to wrap datastorage API
    '''

    _DATASTORAGES = 'api/rest/v1/datastorage'
    _DATASTORAGE = 'api/rest/v1/datastorage/{}'
    _DATASTORAGE_DATA = 'api/rest/v1/datastorage/{}/data'
    _DATASTORAGE_DATA_KEY = 'api/rest/v1/datastorage/{}/data/{}'

    def __init__(self, credentials):
        self._server = credentials.server

        if credentials.oidc_client is None:
            self._session = requests
            self._auth = credentials.auth
        else:
            self._session = credentials.oidc_client
            self._auth = None

        self._auth_header = credentials.access_token

    def get_all(self,
                dsfilter=None,
                datafilter=None,
                start_index=0,
                count=100):
        '''Get the list of the datastorages.
        Args:
            dsfilter (Filter)      : filter out by datastorage fields.
            datafilter (Filter)    : filter out by data keys and values.
            start_index (int)      : 0-based pagination start index
            count (int)            : pagination page size, max 500
        Returns:
            Page with the pagination data and the list of datastorages.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        params = {}

        if (dsfilter != None):
            params['dsfilter'] = dsfilter.generate()

        if (datafilter != None):
            params['datafilter'] = datafilter.generate()

        if (start_index != None):
            params['startIndex'] = start_index

        if (count != None):
            params['count'] = count

        response = self._session.get(su.uri(self._server, self._DATASTORAGES),
                                params=params,
                                headers=self._auth_header,
                                auth=self._auth)

        if response.status_code is 200:
            logging.debug('Found {} datastorages'.format(
                                    len(response.json()['resources'])))
            datastorages = []
            for ds in response.json()['resources']:
                datastorages.append(DataStorage(storage_fields = ds))
            page = Page(response.json(), datastorages)
            return page
        else:
            logging.debug('Failed to fild datastorages')
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def create(self, datastorage):
        '''Create new Data Storage.
        Args:
            datastorage (DataStorage) : data storage to create. Must contain
                                        name. If data is defined, it will be
                                        added to the new datastorage.
        Returns:
            New data storage object.
        Raises:
            TrivoreIDException if the status code is not 201.
        '''
        if 'DataStorage' not in str(type(datastorage)):
            raise TrivoreIDSDKException('DataStorage type is wrong!')

        if bool(datastorage.name) is False:
                raise TrivoreIDSDKException('Datastorage name was not defined!')

        response = self._session.post(su.uri(self._server, self._DATASTORAGES),
                                json=datastorage.serialize(),
                                headers=self._auth_header,
                                auth=self._auth)

        if response.status_code == 201:
            new_ds = DataStorage(storage_fields=response.json())
            logging.debug('Successfully created datastorage with id {}'.format(
                                new_ds.id))

            if datastorage.data is not None:
                self.update_data(new_ds.id, datastorage.data)

            return new_ds
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def get(self, dsId):
        '''Get the datastorage settings by id.
        Args:
            dsId (str) : datasrorage unique identifier
        Returns:
            Datastorage object (settings only).
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        response = self._session.get(
                            su.uri(self._server, self._DATASTORAGE).format(dsId),
                            headers=self._auth_header,
                            auth=self._auth)

        if response.status_code is 200:
            logging.debug('Found datastorage with id {}'.format(dsId))
            return DataStorage(storage_fields=response.json())
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def update(self, datastorage):
        '''Update existing datastorage.
        Args:
            datastorage (DataStorage) : Update datastorage data and settings.
        Returns:
            Modified datastorage object (settings only).
        Raises:
            TrivoreIDException if the status code is not 200.
        '''

        if 'DataStorage' not in str(type(datastorage)):
            raise TrivoreIDSDKException('DataStorage type is wrong!')

        if bool(datastorage.id) is False:
            raise TrivoreIDSDKException('DataStorage id was not defined!')

        response = self._session.put(su.uri(self._server, self._DATASTORAGE)
                                                    .format(datastorage.id),
                                    json = datastorage.serialize(),
                                    headers=self._auth_header,
                                    auth=self._auth)

        if response.status_code is 200:
            storage = DataStorage(storage_fields=response.json())
            logging.debug('Successfully updated datastorage with id {}'.format(storage.id))

            if datastorage.data != None:
                self.update_data(storage.id, datastorage.data)

            return storage
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def delete(self, dsId):
        '''Delete existing datastorage.
        Args:
            dsId (sr) : datastorage ID to delete
        Returns:
            Modified datastorage object (settings only).
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        response = self._session.delete(su.uri(self._server, self._DATASTORAGE)
                                                    .format(dsId),
                                    headers=self._auth_header,
                                    auth=self._auth)

        if response.status_code is 204:
            logging.debug('Deleted datastorage with id {}'.format(dsId))
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def get_data(self, dsId):
        '''Get data from the datastorage.
        Args:
            dsId (str) : datastorage unique identifier
        Returns:
            Json data.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        response = self._session.get(
                su.uri(self._server, self._DATASTORAGE_DATA).format(dsId),
                headers=self._auth_header,
                auth=self._auth)

        if response.status_code is 200:
            logging.debug('Found data in the datastorage with id {}'.format(dsId))
            return response.content
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def update_data(self, dsId, data):
        '''Update data in the datastorage.
        Args:
            dsId (str)  : datastorage unique identifier
            data (dict) : data to update
        Raises:
            TrivoreIDException if the status code is not 204.
        '''
        response = self._session.put(
                    su.uri(self._server, self._DATASTORAGE_DATA).format(dsId),
                    json = data,
                    headers=self._auth_header,
                    auth=self._auth)

        if response.status_code != 204:
            raise TrivoreIDException(
                                su.error_response_message(response),
                                response.status_code)
        else:
            logging.debug('Successfully updated the data in the datastorage with id {}'
                                        .format(dsId))

    def get_value(self, dsId, key):
        '''Get a single value from the datastorage by the key.
        Args:
            dsId (str) : datastorage unique identifier
        Returns:
            Json data.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        uri = su.uri(self._server, self._DATASTORAGE_DATA_KEY).format(dsId, key)
        response = self._session.get(uri,
                                    headers=self._auth_header,
                                    auth=self._auth)

        if response.status_code is 200:
            logging.debug('Found value in the datastorage with id {} and key {}'
                                                            .format(dsId, key))
            return json.loads(response.content)['value']
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def update_value(self, dsId, key, value):
        '''Update a single value value in the datastorage by key.
        Args:
            dsId (str)  : datastorage unique identifier
            data (dict) : data to update
        Raises:
            TrivoreIDException if the status code is not 204.
        '''
        response = self._session.put(
             su.uri(self._server, self._DATASTORAGE_DATA_KEY).format(dsId, key),
                       json ={'value' : value},
                       headers=self._auth_header,
                       auth=self._auth)

        if response.status_code == 204:
            logging.debug('Successfully updated a single value in'
                    +' the datastorage with id {} and key {}'.format(dsId, key))
        else:
            raise TrivoreIDException(
                                su.error_response_message(response),
                                response.status_code)

    def delete_value(self, dsId, key):
        '''Delete/clear single value by key.
        Args:
            dsId (str) : datastorage ID
            key (str)  : the key to clear value from
        Returns:
            Modified datastorage object (settings only).
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        response = self._session.delete(
                        su.uri(self._server, self._DATASTORAGE_DATA_KEY)
                                        .format(dsId, key),
                       headers=self._auth_header,
                       auth=self._auth)

        if response.status_code is 204:
            logging.debug('Deleted value from the datastorage with id {} and the key {}'
                                                    .format(dsId, key))
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)



#!/usr/bin/env python
# coding: utf-8

from trivoreid.models.misc import Meta

class Namespace(object):
    '''
    Class that represents a namespace object.
    '''

    def __init__(self, data={}):
        '''
        Args:
            data (dict): namespace fields
        Dictionary keys:
            'id' (str) - namespace ID
            'code' (str)
            'name' (str)
            'shortName' (str)
            'usernamePolicy' (UsernamePolicy) - if None, EIGHT_NUMBERS by defalt
            'commMethodMaxQty' Maximum number of email addresses and mobile
                               phone numbers allowed for users
            'duplicateNicknamesAllowed' Sets whether duplicate nicknames are
                                        allowed within the namespace
            'validFrom'
            'validTo'
            'smsSettings'
            'defaultGroupPolicy'
            'authorisationRestrictedMode' Restricted mode for the authoristions
                                          enabled. Can only be enabled in the
                                          Web UI.
            'meta' (dict) meta data
        '''

        self.id = data.pop('id', None)
        self.code = data.pop('code', None)
        self.name = data.pop('name', None)
        self.shortName = data.pop('shortName', None)
        self.usernamePolicy = data.pop('usernamePolicy', UsernamePolicy.EIGHT_NUMBERS)
        self.commMethodMaxQty = data.pop('commMethodMaxQty', 2)
        self.duplicateNicknamesAllowed = data.pop('duplicateNicknamesAllowed', False)
        self.validFrom = data.pop('validFrom', None)
        self.validTo = data.pop('validTo', None)
        self.smsSettings = SMSSettings(data.pop('smsSettings', {}))
        self.defaultGroupPolicy = DefaultGroupPolicy(data.pop('defaultGroupPolicy', {}))
        self.authorisationRestrictedMode = data.pop('authorisationRestrictedMode', None)
        self.meta = Meta(data.pop('meta', {}))

    def serialize(self, creating=False):
        '''
        Return JSON serializable dictionary with namespace fields.
        Args:
            creating If the new namespace is created. Default is False.
        '''

        namespace = {
            'id'                            : self.id,
            'code'                          : self.code,
            'name'                          : self.name,
            'shortName'                     : self.shortName,
            'usernamePolicy'                : self.usernamePolicy,
            'commMethodMaxQty'              : self.commMethodMaxQty,
            'duplicateNicknamesAllowed'     : self.duplicateNicknamesAllowed,
            'validFrom'                     : self.validFrom,
            'validTo'                       : self.validTo,
            'smsSettings'                   : self.smsSettings.serialize(),
            'authorisationRestrictedMode'   : self.authorisationRestrictedMode,
            'meta'                          : self.meta.serialize()
        }

        if not creating:
            namespace['defaultGroupPolicy'] = self.defaultGroupPolicy.serialize()

        return namespace

class UsernamePolicy:
    '''
    Namespace's username policies.
    '''
    EMAIL = 'EMAIL'
    FMLNN = 'FMLNN'
    CVCV = 'CVCV'
    CVCVCV = 'CVCVCV'
    EIGHT_NUMBERS = 'EIGHT_NUMBERS'
    NINE_NUMBERS = 'NINE_NUMBERS'
    TEN_NUMBERS = 'TEN_NUMBERS'
    EIGHT_LOWER_CASE_CHARS = 'EIGHT_LOWER_CASE_CHARS'
    TEN_LOWER_CASE_CHARS = 'TEN_LOWER_CASE_CHARS'
    MANUAL = 'MANUAL'

class SMSSettings(object):
    '''
    Class representing SMS settings for the namespace.
    '''

    def __init__(self, data={}):
        '''
        Args:
            data (dict): SMS setting fields
        Dictionary keys:
            'enabled' (boolean) - SMS sending enables for the namespace
            'customEnabled' (boolean) Custom SMS sending enabled for the
                                      namespace. Custom SMS messages may be sent
                                      via the REST API or the Web UI.
            'defaultOriginator' (str) Alphanumeric Default originator for the
                                      SMS messages
            'defaultRegion' (str) Default region for parsing and validation.
                                  Represented by a two letter country code,
                                  eg. FI.
        '''

        self.enabled = data.pop('enabled', False)
        self.customEnabled = data.pop('customEnabled', False)
        self.defaultOriginator = data.pop('defaultOriginator', None)
        self.defaultRegion = data.pop('defaultRegion', None)

    def serialize(self):
        return {
            'enabled'           : self.enabled,
            'customEnabled'     : self.customEnabled,
            'defaultOriginator' : self.defaultOriginator,
            'defaultRegion'     : self.defaultRegion
        }

class DefaultGroupPolicy(object):
    '''
    Default group policy of the namespace. Sets many of the user settings.
    '''

    def __init__(self, data={}):
        '''
        Args:
            data (dict): SMS setting fields
        Dictionary keys:
            'policyName' (str)
            'policyDescription' (str)
            'inactivityTime' (int)
            'resetPasswordOptions' (dict)
            'preferredLocale' (str)
            'preferredTimeFormat' (str) see PreferredTimeFormat
            'preferredDateFormat' (str) see PreferredDateFormat
            'displayNameFormat' (str) see DisplayNameFormat
            'timeZone' (str)
            'minPwLength' (int)
            'minCharClasses' (int)
            'pwMaxAge' (int)
            'pwHistorySaved' (int)
            'incorrectLoginAttemptsThresh' (int)
            'incorrectLoginTimeWin' (int)
            'incorrectLoginLockoutTime' (int)
            'maxPwSequenceLength' (int)
            'maxConsecutiveIdenticalChars' (int)
            'dataStoreCountry' (str)
            'passwordUserNameRuleEnforced' (boolean)
            'passwordDictionaryRuleEnforced' (boolean)
            'ciscoGroupPolicy' (str)
            'twoFactorAuthOptions' (str) see TwoFactorAuthOptions
            'loginAllowedIpRules' (list)
            'emailVerifyLinkExpireDays' (int)
            'mobileVerificationRequired' (boolean)
            'persistentLoginAllowed' (boolean)
            'emailLoginAllowed' (str) see IsAllowed
            'mobileLoginAllowed' (str) see IsAllowed
            'mobileVerificationAttemptsPerDay' (int)
            'strongIdRequiredForMyDataDownload' (boolean)
            'emailVerificationUnlocksUser' (boolean)
            'userImageAllowed' (boolean)
            'deleteMode' (str) see DeleteMode
            'hideOnSoftDelete' (boolean)
            'loginTargetBlacklist' (list)
        '''

        loginAllowedIpRules = []
        for l in data.pop('loginAllowedIpRules', []):
            loginAllowedIpRules.append(LoginAllowedIpRule(l))

        self.policyName = data.pop('policyName', None)
        self.policyDescription = data.pop('policyDescription', None)
        self.inactivityTime = data.pop('inactivityTime', None)
        self.resetPasswordOptions = ResetPasswordOptions(data.pop('resetPasswordOptions', {}))
        self.preferredLocale = data.pop('preferredLocale', None)
        self.preferredTimeFormat = data.pop('preferredTimeFormat', None)
        self.preferredDateFormat = data.pop('preferredDateFormat', None)
        self.displayNameFormat = data.pop('displayNameFormat', None)
        self.timeZone = data.pop('timeZone', None)
        self.minPwLength = data.pop('minPwLength', None)
        self.minCharClasses = data.pop('minCharClasses', None)
        self.pwMaxAge = data.pop('pwMaxAge', None)
        self.pwHistorySaved = data.pop('pwHistorySaved', None)
        self.incorrectLoginAttemptsThresh = data.pop('incorrectLoginAttemptsThresh', None)
        self.incorrectLoginTimeWin = data.pop('incorrectLoginTimeWin', None)
        self.incorrectLoginLockoutTime = data.pop('incorrectLoginLockoutTime', None)
        self.maxPwSequenceLength = data.pop('maxPwSequenceLength', None)
        self.maxConsecutiveIdenticalChars = data.pop('maxConsecutiveIdenticalChars', None)
        self.dataStoreCountry = data.pop('dataStoreCountry', None)
        self.passwordUserNameRuleEnforced = data.pop('passwordUserNameRuleEnforced', None)
        self.passwordDictionaryRuleEnforced = data.pop('passwordDictionaryRuleEnforced', None)
        self.ciscoGroupPolicy = data.pop('ciscoGroupPolicy', None)
        self.twoFactorAuthOptions = data.pop('twoFactorAuthOptions', None)
        self.loginAllowedIpRules = loginAllowedIpRules
        self.emailVerifyLinkExpireDays = data.pop('emailVerifyLinkExpireDays', None)
        self.mobileVerificationRequired = data.pop('mobileVerificationRequired', None)
        self.persistentLoginAllowed = data.pop('persistentLoginAllowed', None)
        self.emailLoginAllowed = data.pop('emailLoginAllowed', None)
        self.mobileLoginAllowed = data.pop('mobileLoginAllowed', None)
        self.mobileVerificationAttemptsPerDay = data.pop('mobileVerificationAttemptsPerDay', None)
        self.strongIdRequiredForMyDataDownload = data.pop('strongIdRequiredForMyDataDownload', None)
        self.emailVerificationUnlocksUser = data.pop('emailVerificationUnlocksUser', None)
        self.userImageAllowed = data.pop('userImageAllowed', None)
        self.deleteMode = data.pop('deleteMode', None)
        self.hideOnSoftDelete = data.pop('hideOnSoftDelete', None)
        self.loginTargetBlacklist = data.pop('loginTargetBlacklist', None)

    def serialize(self):

        loginAllowedIpRules = []
        for l in self.loginAllowedIpRules:
            loginAllowedIpRules.append(l.serialize())

        return {
            'policyName'                        : self.policyName,
            'policyDescription'                 : self.policyDescription,
            'inactivityTime'                    : self.inactivityTime,
            'resetPasswordOptions'              : self.resetPasswordOptions.serialize(),
            'preferredLocale'                   : self.preferredLocale,
            'preferredTimeFormat'               : self.preferredTimeFormat,
            'preferredDateFormat'               : self.preferredDateFormat,
            'displayNameFormat'                 : self.displayNameFormat,
            'timeZone'                          : self.timeZone,
            'minPwLength'                       : self.minPwLength,
            'minCharClasses'                    : self.minCharClasses,
            'pwMaxAge'                          : self.pwMaxAge,
            'pwHistorySaved'                    : self.pwHistorySaved,
            'incorrectLoginAttemptsThresh'      : self.incorrectLoginAttemptsThresh,
            'incorrectLoginTimeWin'             : self.incorrectLoginTimeWin,
            'incorrectLoginLockoutTime'         : self.incorrectLoginLockoutTime,
            'maxPwSequenceLength'               : self.maxPwSequenceLength,
            'maxConsecutiveIdenticalChars'      : self.maxConsecutiveIdenticalChars,
            'dataStoreCountry'                  : self.dataStoreCountry,
            'passwordUserNameRuleEnforced'      : self.passwordUserNameRuleEnforced,
            'passwordDictionaryRuleEnforced'    : self.passwordDictionaryRuleEnforced,
            'ciscoGroupPolicy'                  : self.ciscoGroupPolicy,
            'twoFactorAuthOptions'              : self.twoFactorAuthOptions,
            'loginAllowedIpRules'               : loginAllowedIpRules,
            'emailVerifyLinkExpireDays'         : self.emailVerifyLinkExpireDays,
            'mobileVerificationRequired'        : self.mobileVerificationRequired,
            'persistentLoginAllowed'            : self.persistentLoginAllowed,
            'emailLoginAllowed'                 : self.emailLoginAllowed,
            'mobileLoginAllowed'                : self.mobileLoginAllowed,
            'mobileVerificationAttemptsPerDay'  : self.mobileVerificationAttemptsPerDay,
            'strongIdRequiredForMyDataDownload' : self.strongIdRequiredForMyDataDownload,
            'emailVerificationUnlocksUser'      : self.emailVerificationUnlocksUser,
            'userImageAllowed'                  : self.userImageAllowed,
            'deleteMode'                        : self.deleteMode,
            'hideOnSoftDelete'                  : self.hideOnSoftDelete,
            'loginTargetBlacklist'              : self.loginTargetBlacklist
        }

class ResetPasswordOptions(object):
    '''
    Reset password options for the namespace.
    '''

    def __init__(self, data={}):
        '''
        Args:
            data (dict): SMS setting fields
        Dictionary keys:
            'namespaceInfoHidden' (boolean)
            'namespaceInfoHiddenEnabled' (boolean)
            'allFieldsEnabled' (boolean)
        '''
        self.namespaceInfoHidden = data.pop('namespaceInfoHidden', None)
        self.namespaceInfoHiddenEnabled = data.pop('namespaceInfoHiddenEnabled', None)
        self.allFieldsEnabled = data.pop('allFieldsEnabled', None)

    def serialize(self):
        return {
            'namespaceInfoHidden': self.namespaceInfoHidden,
            'namespaceInfoHiddenEnabled': self.namespaceInfoHiddenEnabled,
            'allFieldsEnabled': self.allFieldsEnabled
        }

class LoginAllowedIpRule(object):
    '''
    Login Allowed Ip Rule of the namespace.
    '''

    def __init__(self, data={}):
        '''
        Args:
            data (dict): SMS setting fields
        Dictionary keys:
            'cidr' (str)
            'allowed' (boolean)
        '''
        self.cidr = data.pop('cidr', None)
        self.allowed = data.pop('allowed', None)

    def serialize(self):
        return {
            'cidr': self.cidr,
            'allowed': self.allowed
        }

class PreferredTimeFormat:
    '''
    Preferred time format.
    INHERIT, BROWSER, H_MM_SS_COLON_SEPARATOR
    '''
    INHERIT = 'INHERIT'
    BROWSER = 'BROWSER'
    H_MM_SS_COLON_SEPARATOR = 'H_MM_SS_COLON_SEPARATOR'

class PreferredDateFormat:
    '''
    Preferred date format.
    INHERIT, BROWSER, YYYY_MM_DD, D_M_YYYY, D_MON_YYYY
    '''
    INHERIT = 'INHERIT'
    BROWSER = 'BROWSER'
    H_MM_SS_COLON_SEPARATOR = 'YYYY_MM_DD'
    D_M_YYYY = 'D_M_YYYY'
    D_MON_YYYY = 'D_MON_YYYY'

class DisplayNameFormat:
    '''
    Preferred date format.
    Possible values:
        FIRSTNAME_MIDDLENAME_LASTNAME,
        FIRSTNAME_MIDDLENAME_INITIAL_LASTNAME,
        FIRSTNAME_LASTNAME,
        FIRSTNAME_MIDDLENAME_CAPITAL_LASTNAME,
        FIRSTNAME_MIDDLENAME_INITIAL_CAPITAL_LASTNAME,
        FIRSTNAME_CAPITAL_LASTNAME,
        LASTNAME_COMMA_FIRSTNAME_MIDDLENAME,
        LASTNAME_FIRSTNAME_MIDDLENAME,
        LASTNAME_COMMA_FIRSTNAME_MIDDLENAME_INITIAL,
        LASTNAME_FIRSTNAME_MIDDLENAME_INITIAL,
        LASTNAME_COMMA_FIRSTNAME,
        LASTNAME_FIRSTNAME,
        CAPITAL_LASTNAME_COMMA_FIRSTNAME_MIDDLENAME,
        CAPITAL_LASTNAME_FIRSTNAME_MIDDLENAME,
        CAPITAL_LASTNAME_COMMA_FIRSTNAME_MIDDLENAME_INITIAL,
        CAPITAL_LASTNAME_FIRSTNAME_MIDDLENAME_INITIAL,
        CAPITAL_LASTNAME_COMMA_FIRSTNAME,
        CAPITAL_LASTNAME_FIRSTNAME
    '''
    FIRSTNAME_MIDDLENAME_LASTNAME = 'FIRSTNAME_MIDDLENAME_LASTNAME'
    FIRSTNAME_MIDDLENAME_INITIAL_LASTNAME = 'FIRSTNAME_MIDDLENAME_INITIAL_LASTNAME'
    FIRSTNAME_LASTNAME = 'FIRSTNAME_LASTNAME'
    FIRSTNAME_MIDDLENAME_CAPITAL_LASTNAME = 'FIRSTNAME_MIDDLENAME_CAPITAL_LASTNAME'
    FIRSTNAME_MIDDLENAME_INITIAL_CAPITAL_LASTNAME = 'FIRSTNAME_MIDDLENAME_INITIAL_CAPITAL_LASTNAME'
    FIRSTNAME_CAPITAL_LASTNAME = 'FIRSTNAME_CAPITAL_LASTNAME'
    LASTNAME_COMMA_FIRSTNAME_MIDDLENAME = 'LASTNAME_COMMA_FIRSTNAME_MIDDLENAME'
    LASTNAME_FIRSTNAME_MIDDLENAME = 'LASTNAME_FIRSTNAME_MIDDLENAME'
    LASTNAME_COMMA_FIRSTNAME_MIDDLENAME_INITIAL = 'LASTNAME_COMMA_FIRSTNAME_MIDDLENAME_INITIAL'
    LASTNAME_FIRSTNAME_MIDDLENAME_INITIAL = 'LASTNAME_FIRSTNAME_MIDDLENAME_INITIAL'
    LASTNAME_COMMA_FIRSTNAME = 'LASTNAME_COMMA_FIRSTNAME'
    LASTNAME_FIRSTNAME = 'LASTNAME_FIRSTNAME'
    CAPITAL_LASTNAME_COMMA_FIRSTNAME_MIDDLENAME = 'CAPITAL_LASTNAME_COMMA_FIRSTNAME_MIDDLENAME'
    CAPITAL_LASTNAME_FIRSTNAME_MIDDLENAME = 'CAPITAL_LASTNAME_FIRSTNAME_MIDDLENAME'
    CAPITAL_LASTNAME_COMMA_FIRSTNAME_MIDDLENAME_INITIAL = 'CAPITAL_LASTNAME_COMMA_FIRSTNAME_MIDDLENAME_INITIAL'
    CAPITAL_LASTNAME_FIRSTNAME_MIDDLENAME_INITIAL = 'CAPITAL_LASTNAME_FIRSTNAME_MIDDLENAME_INITIAL'
    CAPITAL_LASTNAME_COMMA_FIRSTNAME = 'CAPITAL_LASTNAME_COMMA_FIRSTNAME'
    CAPITAL_LASTNAME_FIRSTNAME = 'CAPITAL_LASTNAME_FIRSTNAME'

class TwoFactorAuthOptions:
    '''
    Two Factor Authentication Options.
    Values:
        Allow TOTP and SMS OTP 2FA
        Allow TOTP 2FA only
        Request TOPT or SMS OTL 2FA
        Request TOTP 2FA
    '''
    ALLOW_TOTP_AND_SMS_OTL_2FA = 'Allow TOTP and SMS OTP 2FA'
    ALLOW_TOTP_2FA_ONLY = 'Allow TOTP 2FA only'
    REQUEST_TOTP_OR_SMS_OTL_2FA = 'Request TOPT or SMS OTL 2FA'
    REQUEST_TOTP_2FA = 'Request TOTP 2FA'

class IsAllowed:
    '''
    ALLOWED or DISALLOWED.
    '''
    ALLOWED = 'ALLOWED'
    DISALLOWED = 'DISALLOWED'

class DeleteMode:
    '''
    HARD_DELETE or SOFT_DELETE.
    '''
    HARD_DELETE = 'HARD_DELETE'
    SOFT_DELETE = 'SOFT_DELETE'



#!/usr/bin/env python
# coding: utf-8

import requests
import logging

import trivoreid.utils.service_utils as su
from trivoreid.models.profile import Profile
from trivoreid.exceptions import TrivoreIDSDKException, TrivoreIDException

class ProfileService(object):
    '''
    Class to wrap Trivore ID profile API
    '''

    _PROFILE = 'api/rest/v1/user/{}/profile'

    def __init__(self, credentials, oidc_user=None):
        self._server = credentials.server
        self._oidc_user = oidc_user

        if credentials.oidc_client is None:
            self._session = requests
            self._auth = credentials.auth
        else:
            self._session = credentials.oidc_client
            self._auth = None

        self._auth_header = credentials.access_token

    def get(self, userId=None):
        '''Get user's profile.
        Args:
            userId (str) : currently logged in user's unique identifier
        Returns:
            A profile.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        if userId is None:
            userId = self._oidc_user.id

        response = self._session.get(
                        su.uri(self._server, self._PROFILE).format(userId),
                        headers=self._auth_header,
                        auth=self._auth)

        if response.status_code is 200:
            logging.debug('Found profile with user id {}'.format(userId))
            return Profile(response.json())
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def update(self, profile, userId=None):
        '''Update user profile.
        Args:
            userId (str) : user's unique identifier
        Returns:
            Modified user's profile.
        Raises:
            TrivoreIDException if the status code is not 200.
            TrivoreIDSDKException exeption can be raised
        '''

        if 'Profile' not in str(type(profile)):
            raise TrivoreIDSDKException('User type is wrong!')

        if userId is None:
            userId = self._oidc_user.id

        response = self._session.put(
                        su.uri(self._server, self._PROFILE).format(userId),
                        json=profile.serialize(),
                        headers=self._auth_header,
                        auth=self._auth)

        if response.status_code is 200:
            logging.debug('Successfully modified profile with the user id {}'
                                                            .format(userId))
            return Profile(response.json())
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)



#!/usr/bin/env python
# coding: utf-8

class SMSMessage(object):
    '''
    Object to wrap parameters for the SMS message.
    '''
    def __init__(self, data={}):
        '''
        Set parameters for the SMS message.
        Args:
            data (dict) : SMS parameters
        Dictionary fields:
            'to'            : Message recipient mobile phone number. Must be
                              valid phone number.
            'to-name'       : Message recipient display name. Used mostly for
                              user-interface purposes.
            'to-region'     : Message recipient default region (country or
                              subdivision). This affects number parsing only if
                              number is given in national format If this parameter
                              is missing, namespace's default region is used.
            'from'          : Message sender number or alphanumeric name. Maximum
                              length is 11 characters.
            'from-required' : Set message sender either required or optional. If
                              message sender (from) is set but this parameter is
                              false, message may be sent via gateway that does not
                              support sender address.
            'messageClass'  : Message class, one of 0,1,2,3
            'text'          : SMS message textual content.
            'data'          : SMS message binary content. Must be hex-encoded
            'udh'           : SMS message user data header. Must be hex-encoded
            'ttl'           : SMS time to live (validity) in minutes.
            'client-ref'    : Free-form, unique client reference for this message.
            'billing-ref'   : Free-form billing reference for this message.
            'carrier-ref'   : Free-form carrier reference for this message.
            'callback'      : Delivery report callback URL.
        '''
        self.to = data.pop('to', None)
        self.toName = data.pop('toName', None)
        self.toRegion = data.pop('toRegion', None)
        self.fromName = data.pop('from', None)
        self.fromRequired = data.pop('fromRequired', None)
        self.messageClass = data.pop('messageClass', None)
        self.text = data.pop('text', None)
        self.data = data.pop('data', None)
        self.udh = data.pop('udh', None)
        self.ttl = data.pop('ttl', None)
        self.clientRef = data.pop('carrierRef', None)
        self.billingRef = data.pop('billingRef', None)
        self.carrierRef = data.pop('carrierRef', None)
        self.callback = data.pop('callback', None)

    def serialize(self):
        '''
        Make object JSON serializable.
        '''
        return {
            'to'            : self.to,
            'toName'        : self.toName,
            'toRegion'      : self.toRegion,
            'from'          : self.fromName,
            'fromRequired'  : self.fromRequired,
            'messageClass'  : self.messageClass,
            'text'          : self.text,
            'data'          : self.data,
            'udh'           : self.udh,
            'ttl'           : self.ttl,
            'clientRef'     : self.clientRef,
            'billingRef'    : self.billingRef,
            'carrierRef'    : self.carrierRef,
            'callback'      : self.callback
        }

class SMSResponse(object):
    '''
    Class to wrap SMS response body.
    '''
    def __init__(self, data):
        self.response = {
            'status'            : data.pop('status', None),
            'description'       : data.pop('description', None),
            'action'            : data.pop('action', None),
            'messageId'         : data.pop('messageId', None),
            'to'                : data.pop('to', None),
            'toRegion'          : data.pop('toRegion', None),
            'clientRef'         : data.pop('clientRef', None),
            'billingRef'        : data.pop('billingRef', None),
            'carrierRef'        : data.pop('carrierRef', None),
            'messageCount'      : data.pop('messageCount', None),
            'billingState'      : data.pop('billingState', None),
            'totalPrice'        : data.pop('totalPrice', None),
            'remainingCredits'  : data.pop('remainingCredits', None)
        }



#!/usr/bin/env python
# coding: utf-8

class Meta(object):
    '''
    Class that represents a Meta Data in TrivoreID.
    '''

    def __init__(self, data={}):
        '''
        Args:
            data (dict)         : group fields
        Dictionary keys:
            'created' (str) time when resource was created
            'lastModified' (str) time when resource was last modified
            'location' (str) resource's location URI
        '''
        self.created = data.pop('created', None)
        self.lastModified = data.pop('lastModified', None)
        self.location = data.pop('location', None)

    def serialize(self):
        return {
            'created'      : self.created,
            'lastModified' : self.lastModified,
            'location'     : self.location
        }



#!/usr/bin/env python
# coding: utf-8

import requests
import logging

import trivoreid.utils.service_utils as su
from trivoreid.utils.criteria import Filter
from trivoreid.models.namespace import Namespace
from trivoreid.models.groups import Group
from trivoreid.models.page import Page
from trivoreid.exceptions import TrivoreIDSDKException, TrivoreIDException

class NamespaceService(object):
    '''
    Class to wrap /namespace API
    '''

    _NAMESPACES = 'api/rest/v1/namespace'
    _NAMESPACE = 'api/rest/v1/namespace/{}'

    def __init__(self, credentials, group_service):
        self._group_service = group_service
        self._server = credentials.server

        if credentials.oidc_client is None:
            self._session = requests
            self._auth = credentials.auth
        else:
            self._session = credentials.oidc_client
            self._auth = None

        self._auth_header = credentials.access_token

    def get_all(self,
                filter_fields = None,
                start_index = 0,
                count = 100,
                all_namespaces=True):
        '''Get the list of the namespaces.
        Args:
            all_namespaces (bool)  : set True to list all namespaces. When
                                     False only accessible namespaces will be
                                     listed
            filter_fields (Filter) : utils.Filter class to filter out
                                     the result.
            start_index (int)      : 0-based pagination start index
            count (int)            : pagination page size, max 500,
                                     100 by default
        Returns:
            Page with the pagination data and the list of namespaces.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''

        params = su.generate_parameters(filter_fields, start_index, count)
        params['all'] = all_namespaces

        response = self._session.get(su.uri(self._server, self._NAMESPACES),
                                params=params,
                                headers=self._auth_header,
                                auth=self._auth)

        if response.status_code == 200:
            logging.debug('Found {} namespaces'.format(
                        len(response.json()['resources'])))
            namespaces = []
            for n in response.json()['resources']:
                namespaces.append(Namespace(n))
            page = Page(response.json(), namespaces)
            return page
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def create(self, namespace):
        '''Create new namespace.
        Args:
            namespace (Namespace) : namespace to create.
        Returns:
            New namespace object.
        Raises:
            TrivoreIDSDKException if the type of the Namespace is wrong
            TrivoreIDException if the status code is not 201.
        '''
        if 'Namespace' not in str(type(namespace)):
            raise TrivoreIDSDKException('Namespace type is wrong!')

        response = self._session.post(su.uri(self._server, self._NAMESPACES),
                                json=namespace.serialize(True),
                                headers=self._auth_header,
                                auth=self._auth)

        if response.status_code == 201:
            ns = Namespace(response.json())
            logging.debug('Successfully created namespace with id {}'.format(
                                ns.id))
            return ns
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def update(self, namespace):
        '''Modify existing namespace.
        Args:
            namespace (Namespace) : namespace to modify. must contain id or
                                    code of the namespace.
        Returns:
            Modified namespace object.
        Raises:
            TrivoreIDSDKException if the type of the Namespace is wrong
            TrivoreIDException if the status code is not 200.
        '''
        if 'Namespace' not in str(type(namespace)):
            raise TrivoreIDSDKException('Namespace type is wrong!')

        if ('id' and 'code' not in namespace.serialize().keys()
                or (bool(namespace.id) is False
                    and bool(namespace.code) is False)):
            raise TrivoreIDSDKException('Should be defined code or ID!')

        if namespace.id is None:
            idOrCode = namespace.code
        else:
            idOrCode = namespace.id

        response = self._session.put(
                         su.uri(self._server, self._NAMESPACE).format(idOrCode),
                         json=namespace.serialize(True),
                         headers=self._auth_header,
                         auth=self._auth)

        if response.status_code == 200:
            ns = Namespace(response.json())
            logging.debug('Successfully modified namespace with id {}'.format(
                                ns.id))
            return ns
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def get(self, idOrCode):
        '''Get the single namespace by its id or code.
        Args:
            idOrCode (str) : namespace's unique identifier or code
        Returns:
            A namespace.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        response = self._session.get(
                    su.uri(self._server, self._NAMESPACE).format(idOrCode),
                    headers=self._auth_header,
                    auth=self._auth)

        if response.status_code == 200:
            logging.debug('Found namespace with {}'.format(idOrCode))
            return Namespace(response.json())
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def check_namespace_for_groups(self, nsCode, group_names):
        '''
        Check if namespace has all defined groups. If not - create.
        Args:
            nsCode (str)       : namespace code
            group_names (list) : list of group names to check
        '''
        if type(group_names) is str:
            group_names = [group_names]

        ### Check if namespace exists
        namespaces = self.get_all(Filter(Filter.EQUAL, 'code', nsCode)).resources

        if len(namespaces) is 0:
            raise TrivoreIDException(
                    'There is no namespace with the nsCode {}'.format(nsCode))
        elif len(namespaces) is 1:
            group_list = self._group_service.get_all(
                                Filter(Filter.EQUAL, 'nsCode', nsCode)).resources

            for name in group_names:
                if not group_list.check_group_name(name):
                    self._group_service.create(Group({
                            'name' : name,
                            'nsCode' : nsCode
                        }))
        else:
           raise TrivoreIDException(
                'There are more than two namespaces with the code {}'
                            .format(nsCode))

    def delete(self, idOrCode):
        '''Delete namespace by id or code.
        Args:
            idOrCode (str) : namespace id or code
        Raises:
            TrivoreIDException if the status code is not 204.
        '''
        response = self._session.delete(
                    su.uri(self._server, self._NAMESPACE).format(idOrCode),
                    headers=self._auth_header,
                    auth=self._auth)

        if response.status_code is 204:
            logging.debug('Successfully deleted namespace with idOrCode {}'
                                                            .format(idOrCode))
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)



#!/usr/bin/env python
# coding: utf-8

class DataStorage(object):
    '''
    Class that represents a datastorage object.
    '''

    def __init__(self, storage_fields={}, data=None):
        '''
        Args:
            data (dict)           : Data storage data. Any model.
            storage_fields (dirt) : Data storage fields
        Dictionary keys for the storage fields:
            'id' - Data Storage ID
            'name' - Data Storage name
            'description' description
            'ownerId' - Data Storage owner ID
        '''
        self.id = storage_fields.pop('id', None)
        self.description = storage_fields.pop('description', None)
        self.owner_id = storage_fields.pop('ownerId', None)
        self.name = storage_fields.pop('name', None)
        self.admin_access = storage_fields.pop('adminAccess', None)
        self.read_access = storage_fields.pop('readAccess', None)
        self.write_access = storage_fields.pop('writeAccess', None)
        self.size = storage_fields.pop('size', None)

        self.data = data

    def serialize(self):
        '''
        Returns JSON serializable object.
        '''
        return {
            'id'            : self.id,
            'description'   : self.description,
            'owner_id'      : self.owner_id,
            'name'          : self.name,
            'readAccess'    : self.read_access,
            'writeAccess'   : self.write_access,
            'adminAccess'   : self.admin_access,
            'size'          : self.size
        }



#!/usr/bin/env python
# coding: utf-8

import requests
import logging

import trivoreid.utils.service_utils as su
from trivoreid.utils.criteria import Filter
from trivoreid.exceptions import TrivoreIDSDKException, TrivoreIDException

class EmailService(object):
    '''
    Wrapper for sending email messages through Trivore ID api.
    '''
    _SEND = 'api/rest/v1/email/send'
    _USER_SEND = 'api/rest/v1/user/{}/email/send'
    _GROUP_SEND = 'api/rest/v1/group/{}/email/send'

    def __init__(self,
                 credentials,
                 group_service=None,
                 user_service=None):
        self._group_service = group_service
        self._user_service = user_service
        self._server = credentials.server

        if credentials.oidc_client is None:
            self._session = requests
            self._auth = credentials.auth
        else:
            self._session = credentials.oidc_client
            self._auth = None

        self._auth_header = credentials.access_token

    def send(self, email):
        '''
        Send email message.
        Args:
            email (Email) : email to send
        Raises:
            TrivoreIDException if the status code is not 200.
            TrivoreIDSDKException if the type of the Email is wrong
        '''
        if 'Email' not in str(type(email)):
            raise TrivoreIDSDKException('Email type is wrong!')

        response = self._session.post(su.uri(self._server, self._SEND),
                                      json=email.serialize(),
                                      headers=self._auth_header,
                                      auth=self._auth)

        if response.status_code == 200:
            logging.debug('Successfully sent email message.')
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def send_to_user(self, email, userId):
        '''
        Send email message to the user.
        Args:
            email (Email) : email to send
            userId (str)  : user unique identifier
        Raises:
            TrivoreIDSDKException if the type of the Email is wrong
            TrivoreIDException if the status code is not 200.
        '''
        if 'Email' not in str(type(email)):
            raise TrivoreIDSDKException('Email type is wrong!')

        uri = su.uri(self._server, self._USER_SEND).format(userId)
        response = self._session.post(uri,
                                      json=email.serialize(),
                                      headers=self._auth_header,
                                      auth=self._auth)

        if response.status_code == 200:
            logging.debug('Successfully sent email message to a user with id {}.'
                                            .format(userId))
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def send_to_group_members(self, email, groupId):
        '''
        Send email message to all users belonging to the defined groups.
        Args:
            email (Email) : email to send
            groupId (str) : group id.
        '''
        if 'Email' not in str(type(email)):
            raise TrivoreIDSDKException('Email type is wrong!')

        uri = su.uri(self._server, self._GROUP_SEND).format(groupId)
        response = self._session.post(uri,
                                      json=email.serialize(),
                                      headers=self._auth_header,
                                      auth=self._auth)

        if response.status_code == 200:
            logging.debug('Successfully sent email message to a group with id {}.'
                                                .format(groupId))
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)



#!/usr/bin/env python
# coding: utf-8

import requests
import logging

import trivoreid.utils.service_utils as su
from trivoreid.models.mydata import MyDataPackage
from trivoreid.exceptions import TrivoreIDSDKException, TrivoreIDException

class MyDataService(object):

    _MYDATA = 'api/rest/v1/user/{}/mydata'
    _MYDATA_FILE = 'api/rest/v1/user/{}/mydata/{}'
    _USER_MYDATA = 'api/rest/v1/user/{}/mydata/user'

    # not covered
    _DATA_REQUESTS = 'api/rest/v1/user/{}/personaldatarequests'
    _DATA_REQUEST = 'api/rest/v1/user/{}/personaldatarequests/{}'
    _DATA_REQUEST_URI = 'api/rest/v1/user/{}/personaldatarequests/view-uri'

    def __init__(self, credentials):
        self._server = credentials.server

        if credentials.oidc_client is None:
            self._session = requests
            self._auth = credentials.auth
        else:
            self._session = credentials.oidc_client
            self._auth = None

        self._header = credentials.access_token

    def get_package(self, userId=None):
        '''Get package of user MyData entries.
        Args:
            userId (str) : user's ID. If None, then the ID of the OIDC user
                           will be used.
        Returns:
            MyDataPackage object.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        if userId is None:
            userId = self._oidc_client.user.id

        response = self._session.get(
                        su.uri(self._server, self._MYDATA).format(userId),
                        headers=self._header,
                        auth=self._auth)

        if response.status_code == 200:
            logging.debug('Found MyData of the user with id {}'.format(userId))
            return MyDataPackage(response.json())
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def request(self, userId=None):
        '''Request MyData creation for the user.
        Args:
            userId (str) : user's ID. If None, then the ID of the OIDC user
                           will be used.
        Returns:
            MyDataPackage object.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        if userId is None:
            userId = self._oidc_client.user.id

        response = self._session.post(
                        su.uri(self._server, self._MYDATA).format(userId),
                        headers=self._header,
                        auth=self._auth)

        if response.status_code == 200:
            logging.debug('Successfully requested MyData creation for the user {}'
                                                            .format(userId))
            return MyDataPackage(response.json())
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def get_user_info(self, userId=None):
        '''Get user personal data.
        Args:
            userId (str) : user's ID. If None, then the ID of the OIDC user
                           will be used.
        Returns:
            User personal data.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        if userId is None and self._oidc_client is not None:
            if self._oidc_client.user is not None:
                userId = self._oidc_client.user.id

        uri = su.uri(self._server, self._USER_MYDATA).format(userId)
        response = self._session.get(uri, headers=self._header, auth=self._auth)

        if response.status_code == 200:
            logging.debug('Successfully found personal data of the user {}'
                                                            .format(userId))
            return response.json()
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def get_mydata(self, userId, dataId):
        '''Get a single file from the MyData package.
        Args:
            userId (str) : user id
            dataId (str) : data id
        Returns:
            User personal data.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        uri = su.uri(self._server, self._MYDATA_FILE).format(userId, dataId)
        response = self._session.get(uri, headers=self._header, auth=self._auth)

        if response.status_code == 200:
            logging.debug('Successfully found mydata file () of the user {}'
                                                        .format(dataId, userId))
            return response
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)



#!/usr/bin/env python
# coding: utf-8

class Page(object):
    '''
    A page wrapper to wrap Trivore ID specific list results.
    '''
    def __init__(self, data, resources):
        '''
        Args:
            data (dict): the query result.
        Dictionary keys:
            'totalResults' (int) : Total number of possible results, ignoring
                                   pagination.
            'startIndex' (int)   : Start index, if pagination parameters
                                   were used.
            'itemsPerPage' (int) : Number of items per page, if pagination
                                   parameters were used.
            'resources' (list)   : list of Found resources.
        '''

        self.totalResults = data.pop('totalResults', None)
        self.startIndex = data.pop('startIndex', None)
        self.itemsPerPage = data.pop('itemsPerPage', None)
        self.resources = resources



#!/usr/bin/env python
# coding: utf-8

from trivoreid.models.user import Names, Mobile, Address, Consents
from trivoreid.models.email import EmailAddress

class Profile(object):

    def __init__(self, data={}):
        '''
        Args:
            data (dict): profile fields
        Dictionary keys:
            'addresses'
            'legalAddresses'
            'emails'
            'mobiles'
            'name'
            'nickName'
            'dateOfBirth'
            'locale'
            'consents'
            'minor' (bool) Is user a legal minor (client defined age). Can be
                           used as alternative to dateOfBirth.
            'timeZone'
            'lastModified'
            'domicileCode'
            'legalDomicileCode'
        '''
        self.mobiles = []
        for m in data.pop('mobiles', []):
            self.mobiles.append(ProfileMobile(m))

        self.addresses = []
        for a in data.pop('addresses', []):
            self.addresses.append(Address(a))

        self.legal_addresses = []
        for l in data.pop('addresses', []):
            self.legal_addresses.append(Address(l))

        self.emails = []
        for e in data.pop('emails', []):
            self.emails.append(ProfileEmail(e))

        self.name = Names(data.pop('name', {}))
        self.nickName = data.pop('nickName', None)
        self.dateOfBirth = data.pop('dateOfBirth', None)
        self.locale = data.pop('locale', None)
        self.consents = Consents(data.pop('consents', {}))
        self.minor = data.pop('minor', None)
        self.timeZone = data.pop('timeZone', None)
        self.lastModified = data.pop('lastModified', None)
        self.domicileCode = data.pop('domicileCode', None)
        self.legalDomicileCode = data.pop('legalDomicileCode', None)

    def serialize(self):
        '''
        Return dictionary that is JSON serializable.
        '''

        mobiles = []
        for m in self.mobiles:
            mobiles.append(m.serialize())

        addresses = []
        for a in self.addresses:
            addresses.append(a.serialize())

        legal_addresses = []
        for a in self.addresses:
            legal_addresses.append(a.serialize())

        emails = []
        for e in self.emails:
            emails.append(e.serialize())

        return {
            'dateOfBirth'       : self.dateOfBirth,
            'nickName'          : self.nickName,
            'emails'            : emails,
            'mobiles'           : mobiles,
            'name'              : self.name.serialize(),
            'locale'            : self.locale,
            'addresses'         : addresses,
            'legalAddresses'    : legal_addresses,
            'consents'          : self.consents.serialize(),
            'minor'             : self.minor,
            'timeZone'          : self.timeZone,
            'lastModified'      : self.lastModified,
            'domicileCode'      : self.domicileCode,
            'legalDomicileCode' : self.legalDomicileCode
        }

class ProfileMobile(object):
    '''
    Wrapper for a single profile mobile.
    '''
    def __init__(self, data={}):
        '''
        Args:
            data (dict, str) : mobile fields, or single mobile number.
        Dictionary keys:
            'number' (str) - mobile number
            'verified' (boolean) - verification status
        '''
        self.number = data.pop('number', None)
        self.verified = data.pop('verified', False)

    def serialize(self):
        '''
        Return dictionary that is JSON serializable.
        '''
        return {
            'number'    : self.number,
            'verified'  : self.verified
        }

class ProfileEmail(object):
    '''
    Wrapper for a single profile email.
    '''
    def __init__(self, data={}):
        '''
        Args:
            data (dict, str): email fields or a single email address
        Dictionary keys:
            'address'       : email address
            'verified'      : verification status
        '''
        self.address = data.pop('address', None)
        self.verified = data.pop('verified', False)

    def serialize(self):
        '''
        Return dictionary that is JSON serializable.
        '''
        return {
            'address'       : self.address,
            'verified'      : self.verified
        }



#!/usr/bin/env python
# coding: utf-8

import requests
import os
import base64

import trivoreid.utils.service_utils as su

from trivoreid.exceptions import TrivoreIDException
from trivoreid.oidc.oidc_user import OIDCUser


class OidcClient(object):
    '''
    Helper to create OIDC client.
    '''

    _OPENID_CONFIG = '.well-known/openid-configuration'
    _ACCESS_TOKEN = 'openid/token'

    def __init__(self,
                 path='/etc/trivoreid/client_sdk.properties',
                 server=None,
                 scopes=None,
                 client_id=None,
                 client_secret=None,
                 username=None,
                 password=None):
        '''
        Initialize OidcClient.
        Args:
            path (str)          : path to properties file.
                                  Default: /etc/trivoreid/client_sdk.properties
            server (str)        : server URL. If None, value from the properties
                                  file will be used.
            scopes (list)       : list of scopes. If None, then all supported
                                  scopes from the openid-configuration will be
                                  used.
            client_id (str)     : client id. If None, then value from the
                                  property file will be used.
            client_secret (str) : client secret. If None, then value from the
                                  property file will be used.
            username (str)      : username. If None, then value from the
                                  property file will be used.
            password (str)      : password. If None, then value from the
                                  property file will be used.

        Example of the properties file for the password grant:

            service.address=<placeholder>
            oidc.client.id=<placeholder>
            oidc.client.secret=<placeholder>
            password.grant.username=<placeholder>
            password.grant.password=<placeholder>

        '''

        properties = su.get_properties(path)

        if server == None:
            self.server = properties.pop('service.address', None)
        else:
            self.server = server
        if client_id == None:
            self._client_id = properties.pop('oidc.client.id', None)
        else:
            self._client_id = client_id
        if client_secret == None:
            self._client_secret = properties.pop('oidc.client.secret', None)
        else:
            self._client_secret = client_secret
        if username == None:
            self._username = properties.pop('password.grant.username', None)
        else:
            self._username = username
        if password == None:
            self._password = properties.pop('password.grant.password', None)
        else:
            self._password = password

        if self.server[-1] != '/':
            self.server += '/'

        response = requests.get(self.server + self._OPENID_CONFIG)

        if response.status_code is 200:
            response = response.json()
            self.issuer = response['issuer']
            self.authorization_endpoint = response['authorization_endpoint']
            self.jwks_uri = response['jwks_uri']
            self.token_endpoint = response['token_endpoint']
            self.userinfo_endpoint = response['userinfo_endpoint']
            self.scopes_supported = response['scopes_supported']
        else:
            raise TrivoreIDException(self._error_response_message(response),
                                     response.status_code)

        if scopes == None:
            self.scopes = self.scopes_supported
        else:
            self.scopes = scopes

        if type(self.scopes) == list:
            self.scopes = ' '.join(self.scopes)

    def get_access_token(self):
        '''
        Get access token using Password Grant Authorization. Client Id, client
        secret, server, username and password should be defined in the
        constructor.
        '''
        authBytes = base64.encodebytes(
                    ('%s:%s' % (self._client_id, self._client_secret)).encode())
        authString = authBytes.decode().replace('\n', '')

        headers = {'Content-Type': 'application/x-www-form-urlencoded',
                  'Authorization': 'Basic ' + authString}

        data = {'grant_type': 'password',
                'username'  : self._username,
                'password'  : self._password,
                'scope'     : self.scopes}

        token_url = self.server + self._ACCESS_TOKEN
        response = requests.post(token_url, params=data, headers=headers)
        if response.status_code is 200:
            txt = ''.join(c for c in response.text if c not in '{}"').split(',')
            tokens = {}
            for t in txt:
                tokens[t.split(':')[0]] = t.split(':')[1]

            access_token = tokens['access_token']
            headers = {'Authorization': 'Bearer ' + access_token}

            response2 = requests.get(self.userinfo_endpoint, headers=headers)
            if response2.status_code is 200:
                self.user = OIDCUser(response2.json())

            return access_token
        else:
            raise TrivoreIDException(self._error_response_message(response),
                                     response.status_code)

    def _error_response_message(self, response):
        try:
            errorMessage = response.json()['errorMessage']
        except:
            errorMessage = 'Unknown error message. Content: {}'.format(response)

        return 'status code {}, {}'.format(response.status_code, errorMessage)



#!/usr/bin/env python
# coding: utf-8

from trivoreid.exceptions import TrivoreIDSDKException

class Invite(object):
    '''
    The class for constructing an invite request.
    '''

    def __init__(self,
                 emails,
                 nsCode,
                 groups,
                 subject=None,
                 body_text=None,
                 locale='fi'):
        '''
        Args:
            emails (str or list) : One or multiple emails to send the
                                   invitation.
            nsCode (str)         : Namespace code
            groups (str or list) : One or two of the group names or ids. It is
                                   not allowed to assign the Supervisor role
                                   through the invitation request.
            subject (str)        : Subject of the e-mail. If None, the default
                                   email will be sent.
            body_text (str)      : Body text of the e-mail. If None, the default
                                   email will be sent.
            locale (str)         : The language of the default e-mail. Should
                                   be fi, sv or en.
        '''

        if type(emails) is str:
            emails = [emails]

        if type(groups) is str:
            groups = [groups]

        self.request_body = {
                'emails' : emails,
                'nsCode' : nsCode,
                'groups' : groups,
                'locale' : locale
            }

        if subject is not None:
            self.request_body['subject'] = subject

        if body_text is not None:
            self.request_body['bodyText'] = body_text



#!/usr/bin/env python
# coding: utf-8

import requests
import string
import random
import logging

from random import SystemRandom
import trivoreid.utils.service_utils as su
from trivoreid.utils.criteria import Filter
from trivoreid.models.groups import Group
from trivoreid.models.page import Page
from trivoreid.models.pass_requirements import PasswordRequirements
from trivoreid.models.enterprise import Enterprise
from trivoreid.exceptions import TrivoreIDSDKException, TrivoreIDException
from trivoreid.models.user import (User,
                                  StrongIdentification,
                                  PasswordUpdateResult,
                                  LegalInfo,
                                  StudentState)

class UserService(object):
    '''
    Class to wrap Trivore ID /user APIs
    '''

    _USERS = 'api/rest/v1/user'
    _USER = 'api/rest/v1/user/{}'
    _USER_LEGAL = 'api/rest/v1/user/{}/legal'
    _USER_EMAIL_VERIFY = 'api/rest/v1/user/{}/email/verify'
    _USER_SMS_CODE_VERIFY = 'api/rest/v1/user/{}/sms/verify/send'
    _USER_SMS_CODE_VERIFY_CHECK = 'api/rest/v1/user/{}/sms/verify/check/{}'
    _USER_SMS_LINK_VERIFY = 'api/rest/v1/user/{}/sms/verify/sendLink'
    _INVITE_USER = 'api/rest/v1/invite/user'
    _USER_ENTERPRISE = 'api/rest/v1/user/{}/enterprise'
    _USER_CUSTOM_FIELDS = 'api/rest/v1/user/{}/customfields'
    _PASSWORD_REQUIREMENTS = 'api/rest/v1/user/{}/password/requirements'
    _STRONG_IDENTIFICATION = 'api/rest/v1/user/{}/strongidentification'
    _STRONG_ID_HISTORIES = 'api/rest/v1/user/{}/strongidentification/history'
    _STRONG_ID_HISTORY = 'api/rest/v1/user/{}/strongidentification/history/{}'
    _PASSWORD = 'api/rest/v1/user/{}/password'
    _CUSTOM_PERMISSIONS = 'api/rest/v1/user/{}/permissions/custom'
    _EFFECTIVE_PERMISSIONS = 'api/rest/v1/user/{}/permissions/effective'
    _BUILTIN_ROLES = 'api/rest/v1/user/{}/roles/builtin'
    _CUSTOM_ROLES = 'api/rest/v1/user/{}/roles/custom'
    _MIGRATE_NAMESPACE = 'api/rest/v1/user/{}/migratenamespace'
    _STUDENT_STATE = 'api/rest/v1/user/{}/student/state'

    def __init__(self, credentials, group_service):
        self._group_service = group_service
        self._server = credentials.server

        if credentials.oidc_client is None:
            self._session = requests
            self._auth = credentials.auth
        else:
            self._session = credentials.oidc_client
            self._auth = None

        self._header = credentials.access_token

    def get_all(self,
                filter_fields=None,
                start_index=0,
                count=100):
        '''Get the list of the users.
        Args:
            filter_fields (Filter) : filter out the result.
            start_index (int)      : 0-based pagination start index
            count (int)            : pagination page size, max 500
        Returns:
            Page with the pagination data and the list of users.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        params = su.generate_parameters(filter_fields, start_index, count)
        uri = su.uri(self._server, self._USERS)
        response = self._session.get(uri, params=params,
                                          headers=self._header,
                                          auth=self._auth)

        if response.status_code is 200:
            logging.debug('Found {} users'.format(len(response.json()['resources'])))
            users = []
            for user in response.json()['resources']:
                users.append(User(user))
            page = Page(response.json(), users)
            return page
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def create(self,
               user,
               password_length=15,
               password=None):
        '''Create new user.
        Args:
            user (User, list)         : user create.
            password_length (int)      : length of the generated password
            password (str)             : if None, password will be randomly
                                         generated.
        Returns:
            New user object and user's password.
        Raises:
            TrivoreIDException    if the status code is not 201.
            TrivoreIDSDKException if the type of the User is wrong or namespace
                                  code was not defined
        '''
        if 'User' not in str(type(user)) and type(user) != list:
            raise TrivoreIDSDKException('User type is wrong!')

        if password is None:
            pattern = string.ascii_letters + string.digits
            password = ''
            # to avoid repeating letters
            for i in range (int(password_length/2)):
                password += ''.join(random.sample(pattern, 2))
            if password_length % 2 == 1:
                password += random.sample(pattern, 1)[0]

        user.password = password

        uri = su.uri(self._server, self._USERS)
        response = self._session.post(uri, json=user.serialize(),
                                           headers=self._header,
                                           auth=self._auth)

        if response.status_code == 201:
            user = User(response.json())
            logging.debug('Successfully created user with id {}'.format(
                                user.id))
            return user
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def get(self, userId):
        '''Get a single user by id.
        Args:
            userId (str) : user's unique identifier
        Returns:
            A user.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        uri = su.uri(self._server, self._USER).format(userId)
        response = self._session.get(uri, headers=self._header,
                                          auth=self._auth)

        if response.status_code is 200:
            logging.debug('Found user with id {}'.format(userId))
            return User(response.json())
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def update(self, user, userId=None):
        '''Update existing user.
        Args:
            user (User)  : user to modify.
            userId (str) : if None, then user ID from the User object will
                           be used.
        Returns:
            Modified user object.
        Raises:
            TrivoreIDException if the status code is not 200.
            TrivoreIDSDKException if the type of the user is wrong.
        '''
        if 'User' not in str(type(user)):
            raise TrivoreIDSDKException('User type is wrong!')

        # removing user's password
        user.password = None

        if userId is None:
            userId = user.id

        uri = su.uri(self._server, self._USER).format(userId)
        response = self._session.put(uri, json=user.serialize(),
                                          headers=self._header,
                                          auth = self._auth)

        if response.status_code is 200:
            usr = User(response.json())
            logging.debug('Successfully modified user with the id {}'
                        .format(usr.id))
            return usr
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def delete(self, userId):
        '''Delete user by id.
        Args:
            userId (str) : user's unique identifier
        Raises:
            TrivoreIDException if the status code is not 204.
        '''
        uri = su.uri(self._server, self._USER).format(userId)
        response = self._session.delete(uri, headers=self._header,
                                             auth=self._auth)

        if response.status_code is 204:
            logging.debug('Successfully deleted user with id {}'.format(userId))
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def get_legal_info(self, userId):
        '''Get user's legal info.
        Args:
            userId (str) : user's unique identifier
        Returns:
            A user's legal info.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        uri = su.uri(self._server, self._USER_LEGAL).format(userId)
        response = self._session.get(uri, headers=self._header,
                                          auth=self._auth)

        if response.status_code is 200:
            logging.debug('Found ligal info of the user {}'.format(userId))
            return LegalInfo(response.json())
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def verify_email(self,
                     userId,
                     email=None,
                     return_url=None,
                     expiration_days=7,
                     emailTemplateId=None,
                     emailTemplateProperties=None):
        '''
        Send request to verify user's email address.
        Args:
            userId (str) : user's unique identifier
            email (str) : The email address that shoudld be verified.
            return_url (str) : If not specified, user is directed to
                               identification service front page.
            expiration_days (int) : If not specified, current policies or
                                    default value is used.
            emailTemplateId (str) : If a valid email template ID is given, the
                                    template will be used as message content.
                                    Email verification message template can use
                                    the following supported properties.
           emailTemplateProperties (dict) : Additional email template properties
                                            if email template is used.
        '''
        request_body = {}
        if email is not None:
            request_body['value'] = email
        if return_url is not None:
            request_body['returnURL'] = return_url
        if expiration_days is not None:
            request_body['expirationDays'] = expiration_days
        if emailTemplateId is not None:
            request_body['emailTemplateId'] = emailTemplateId
        if emailTemplateProperties is not None:
            request_body['emailTemplateProperties'] = emailTemplateProperties

        uri = su.uri(self._server, self._USER_EMAIL_VERIFY).format(userId)
        response = self._session.post(uri, json=request_body,
                                           headers=self._header,
                                           auth=self._auth)

        if response.status_code is 204:
            logging.debug('Successfully sent the email verification request ' +
                            'to the user with id {}'.format(userId))
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def verify_sms_code(self, userId, mobile=None):
        '''
        Send request to verify user's mobile number.
        Args:
            userId (str)     : user's unique identifier
            mobile (str)     : The mobile number that shoudld be verified.
        '''
        params = {}
        if mobile is not None:
            params['mobile'] = mobile

        uri = su.uri(self._server, self._USER_SMS_CODE_VERIFY).format(userId)
        response = self._session.post(uri, params=params,
                                           headers=self._header,
                                           auth=self._auth)

        if response.status_code is 204:
            logging.debug('Successfully sent the mobile verification code to the' +
                'mobile number {} and  user with id {}'.format(mobile, userId))
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def verify_sms_code_check(self, userId, code):
        '''
        Check user's sms verification code.
        Args:
            userId (str)   : user's unique identifier
            code (str) : The code to check.
        Returns:
            retult (bool) : True if correct and False if incorrect
        '''

        uri = su.uri(self._server, self._USER_SMS_CODE_VERIFY_CHECK
                                                    .format(userId, code))

        response = self._session.post(uri, headers=self._header,
                                           auth=self._auth)

        if response.status_code == 200:
            return True
        elif response.status_code == 404:
            return False
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def verify_sms_link(self,
                        userId,
                        mobile=None,
                        return_url=None,
                        expiration_days=7):
        '''
        Send request to verify user's mobile number.
        Args:
            userId (str)              : user's unique identifier
            mobile (str)          : The mobile number that shoudld be verified.
            return_url (str)      : If not specified, user is directed to
                                    identification service front page.
            expiration_days (int) : If not specified, current policies or
                                    default value is used.
        '''
        request_body = {}
        if mobile is not None:
            request_body['value'] = mobile
        if return_url is not None:
            request_body['returnURL'] = return_url
        if expiration_days is not None:
            request_body['expirationDays'] = expiration_days

        uri = su.uri(self._server, self._USER_SMS_LINK_VERIFY).format(userId)
        response = self._session.post(uri, json=request_body,
                                           headers=self._header,
                                           auth=self._auth)

        if response.status_code is 204:
            logging.debug('Successfully sent the mobile verification link to the' +
                'mobile number {} and  user with id {}'.format(mobile, userId))
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def invite(self, invite):
        '''Invite new user.
        Args:
            invite (Invite) : invite request.
        Raises:
            TrivoreIDException if the status code is not 200.
            TrivoreIDSDKException if the type of the invite is wrong.
        '''
        if 'Invite' not in str(type(invite)):
            raise TrivoreIDSDKException('Invite type is wrong!')

        uri = su.uri(self._server, self._INVITE_USER)
        response = self._session.post(uri, json=invite.request_body,
                                           headers=self._header,
                                           auth=self._auth)

        if response.status_code is 200:
            logging.debug('Successfully sent the invitation request')
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def get_enterprise(self, userId):
        '''Get the enterprise by user's id.
        Args:
            userId (str) : user's unique identifier
        Returns:
            An Enterprise.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''

        uri = su.uri(self._server, self._USER_ENTERPRISE).format(userId)
        response = self._session.get(uri, headers=self._header,
                                          auth=self._auth)

        if response.status_code is 200:
            logging.debug('Found enterprise from user with id {}'.format(userId))
            return Enterprise(response.json())
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def update_enterprise(self, userId, enterprise):
        '''Update user's enterprise.
        Args:
            userId (str)                : user's unique identifier
            enterprise (Enterprise) : enterprise to modify
        Returns:
            Modified Enterprise.
        Raises:
            TrivoreIDException if the status code is not 200.
            TrivoreIDSDKException if the type of the enterprise is wrong.
        '''
        if 'Enterprise' not in str(type(enterprise)):
            raise TrivoreIDSDKException('Enterprise type is wrong!')

        uri = su.uri(self._server, self._USER_ENTERPRISE).format(userId)

        response = self._session.put(uri, json=enterprise.serialize(),
                                          headers=self._header,
                                          auth = self._auth)

        if response.status_code is 200:
            ent = Enterprise(response.json())
            logging.debug('Successfully modified enterprise with the user id {}'
                                        .format(userId))
            return ent
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def get_custom_fields(self, userId):
        '''Get the user custom fields.
        Args:
            userId (str) : user's unique identifier
        Returns:
            Custom fields.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''

        uri = su.uri(self._server, self._USER_CUSTOM_FIELDS).format(userId)
        response = self._session.get(uri, headers=self._header,
                                          auth=self._auth)

        if response.status_code is 200:
            logging.debug('Found custom fields of the user with id {}'.format(userId))
            return response.content
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def update_custom_fields(self, userId, custom_fields):
        '''Update user custom fields.
        Args:
            userId (str)            : user's unique identifier
            custom_fields (dict)    : custom_fields to update. Must be json
                                      serializable.

        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        uri = su.uri(self._server, self._USER_CUSTOM_FIELDS).format(userId)
        response = self._session.post(uri, json=custom_fields,
                                           headers=self._header,
                                           auth=self._auth)

        if response.status_code is 200:
            logging.debug('Successfully modified custom fields of the user id {}'
                                        .format(userId))
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def delete_custom_fields(self, userId):
        '''Delete ALL user custom fields.
        Args:
            userId (str) : user's unique identifier
        Raises:
            TrivoreIDException if the status code is not 204.
        '''
        uri = su.uri(self._server, self._USER_CUSTOM_FIELDS).format(userId)
        response = self._session.delete(uri, headers=self._header,
                                             auth = self._auth)

        if response.status_code is 204:
            logging.debug('Successfully deleted custom fields of the user with id {}'
                                                                .format(userId))
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def get_password_requirements(self, userId):
        '''Get the password requirements.
        Args:
            userId (str) : user's unique identifier
        Returns:
            Password requirements.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        uri = su.uri(self._server, self._PASSWORD_REQUIREMENTS).format(userId)
        response = self._session.get(uri, headers=self._header,
                                          auth=self._auth)


        if response.status_code is 200:
            logging.debug('Found password requirements of the user with id {}'
                                                            .format(userId))
            return PasswordRequirements(response.json())
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def get_strong_identification(self, userId):
        '''Get strong identification info.
        Args:
            userId (str) : user's unique identifier
        Returns:
            Strong identification.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        uri = su.uri(self._server, self._STRONG_IDENTIFICATION).format(userId)
        response = self._session.get(uri, headers=self._header,
                                          auth=self._auth)

        if response.status_code is 200:
            logging.debug('Successfully found strong identification of the user with id {}'
                                                            .format(userId))
            return StrongIdentification(response.json())
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def report_strong_identification(self, userId, personalId, remarks=None):
        '''Report strong identification.
        Args:
            userId (str) : user's unique identifier
            personalId (str) : personal ID code
            remarks (str) : optional additional remarks about identification
        Returns:
            Updated strong identification.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        uri = su.uri(self._server, self._STRONG_IDENTIFICATION).format(userId)

        data = { 'personalId' : personalId, 'remarks' : remarks }

        response = self._session.post(uri, json=data,
                                           headers=self._header,
                                           auth=self._auth)

        if response.status_code is 200:
            logging.debug('Successfully reported strong identification of the user with id {}'
                                                            .format(userId))
            return StrongIdentification(response.json())
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def get_strong_identification_history(self,
                                          userId,
                                          filter_fields=None,
                                          start_index=0,
                                          count=100,
                                          sortBy=None,
                                          ascending=None):
        '''Get the full strong identification history.
        Args:
            userId (str)           : user's unique identifier
            filter_fields (Filter) : utils.Filter class to filter out
                                     the result.
            start_index (int)      : 0-based pagination start index
            count (int)            : pagination page size, max 500,
                                     100 by default
            sortBy (str)           : Sort by attribute name
            ascending (bool)       : Sort direction (ascending or descending)
        Returns:
            List with strong identification objects.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        params = su.generate_parameters(filter_fields, start_index, count)

        if sortBy != None:
            params['sortBy'] = sortBy

        if ascending != None:
            if ascending:
                params['ascending'] = 'ascending'
            else:
                params['ascending'] = 'descending'

        uri = su.uri(self._server, self._STRONG_ID_HISTORIES).format(userId)
        response = self._session.get(uri, params=params,
                                     headers=self._auth_header,
                                     auth=self._auth)

        if response.status_code == 200:
            logging.debug('Successfully got strong identification history.')

            ls = []
            for l in response.json()['resources']:
                ls.append(StrongIdentification(l))
            page = Page(response.json(), ls)
            return page
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def get_strong_identification_history_entry(self, userId, entryId):
        '''Get a single strong identification entry.
        Args:
            userId (str)  : user's unique identifier
            entryId (str) : entry's unique identifier
        Returns:
            Strong identification.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        uri = su.uri(self._server, self._STRONG_ID).format(userId)
        response = self._session.get(uri, headers=self._header,
                                          auth=self._auth)

        if response.status_code is 200:
            logging.debug('Successfully found strong identification of the user with id {}'
                                                            .format(userId))
            return StrongIdentification(response.json())
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def change_password(self, userId, new_password):
        '''Change user's password
        Args:
            userId (str) : user's unique identifier
            new_password (str) : new password
        Returns:
            Updated password result.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        uri = su.uri(self._server, self._PASSWORD).format(userId)

        data = {'newPassword' : new_password}

        response = self._session.put(uri, json=data,
                                          headers=self._header,
                                          auth=self._auth)

        if response.status_code is 200:
            result = PasswordUpdateResult(response.json())
            if result.success:
                logging.debug('Successfully changed the password of the user with id {}'
                                                            .format(userId))
                return result
            else:
                logging.debug('Failed to change the password of the user with id {}'
                                                            .format(userId))
                return result
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def get_custom_permissions(self, userId):
        '''Get custom permissions of the user.
        Args:
            userId (str) : user's unique identifier
        Returns:
            List with custom permissions.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        uri = su.uri(self._server, self._CUSTOM_PERMISSIONS).format(userId)

        response = self._session.get(uri, headers=self._header,
                                          auth=self._auth)

        if response.status_code is 200:
            logging.debug('Successfully found custom permissions of the user with id {}'
                                                            .format(userId))
            return response.json()
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def update_custom_permissions(self, userId, add=None, remove=None):
        '''Get custom permissions of the user.
        Args:
            userId (str)  : user's unique identifier
            add (list)    : list of permissions to add
            remove (list) : list of permissions to remove
        Returns:
            List with updated custom permissions.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        uri = su.uri(self._server, self._CUSTOM_PERMISSIONS).format(userId)

        data = {'add': add, 'remove': remove}

        response = self._session.put(uri, json=data,
                                          headers=self._header,
                                          auth=self._auth)

        if response.status_code is 200:
            logging.debug('Successfully updated custom permissions of the user with id {}'
                                                            .format(userId))
            return response.json()
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def get_effective_permissions(self, userId):
        '''Get effective permissions of the user.
        Args:
            userId (str) : user's unique identifier
        Returns:
            List with effective permissions.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        uri = su.uri(self._server, self._EFFECTIVE_PERMISSIONS).format(userId)

        response = self._session.get(uri, headers=self._header,
                                          auth=self._auth)

        if response.status_code is 200:
            logging.debug('Successfully found effective permissions of the user with id {}'
                                                            .format(userId))
            return response.json()
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def get_builtin_roles(self, userId):
        '''Get builtin roles of the user.
        Args:
            userId (str) : user's unique identifier
        Returns:
            List with builtin roles.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        uri = su.uri(self._server, self._BUILTIN_ROLES).format(userId)

        response = self._session.get(uri, headers=self._header,
                                          auth=self._auth)

        if response.status_code is 200:
            logging.debug('Successfully found builtin roles of the user with id {}'
                                                            .format(userId))
            return response.json()
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def update_builtin_roles(self, userId, add=None, remove=None):
        '''Get builtin roles of the user.
        Args:
            userId (str)  : user's unique identifier
            add (list)    : list of builtin roles to add
            remove (list) : list of builtin roles to remove
        Returns:
            List with updated builtin roles.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        uri = su.uri(self._server, self._BUILTIN_ROLES).format(userId)

        data = {'add': add, 'remove': remove}

        response = self._session.put(uri, json=data,
                                          headers=self._header,
                                          auth=self._auth)

        if response.status_code is 200:
            logging.debug('Successfully updated builtin roles of the user with id {}'
                                                            .format(userId))
            return response.json()
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def get_custom_roles(self, userId):
        '''Get custom roles of the user.
        Args:
            userId (str) : user's unique identifier
        Returns:
            List with custom roles.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        uri = su.uri(self._server, self._CUSTOM_ROLES).format(userId)

        response = self._session.get(uri, headers=self._header,
                                          auth=self._auth)

        if response.status_code is 200:
            logging.debug('Successfully found custom roles of the user with id {}'
                                                            .format(userId))
            return response.json()
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def update_custom_roles(self, userId, add=None, remove=None):
        '''Get custom roles of the user.
        Args:
            userId (str)  : user's unique identifier
            add (list)    : list of custom roles to add
            remove (list) : list of custom roles to remove
        Returns:
            List with updated custom roles.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        uri = su.uri(self._server, self._CUSTOM_ROLES).format(userId)

        data = {'add': add, 'remove': remove}

        response = self._session.put(uri, json=data,
                                          headers=self._header,
                                          auth=self._auth)

        if response.status_code is 200:
            logging.debug('Successfully updated custom roles of the user with id {}'
                                                            .format(userId))
            return response.json()
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def migrate_namespace(self, userId, options):
        '''Migrate partially user to another namespace.

        Warning: Original user will be deleted and another will be created in
        the target namespace. Not all information will be migrated!

        Args:
            userId (str)  : user's unique identifier
            options use trivoreid.models.user.NamespaceMigrationOptions
        Returns:
            Migrated user object.
        Raises:
            TrivoreIDException if the status code is not 200.
            TrivoreIDSDKException if the type of the NamespaceMigrationOptions
                                   is wrong
        '''
        if 'NamespaceMigrationOptions' not in str(type(options)):
            raise TrivoreIDSDKException('NamespaceMigrationOptions type '
                                                                + 'is wrong!')

        uri = su.uri(self._server, self._MIGRATE_NAMESPACE).format(userId)

        response = self._session.post(uri, json=options.serialize(),
                                           headers=self._header,
                                           auth=self._auth)

        if response.status_code is 200:
            logging.debug('Successfully migrated user with id {}'.format(userId))
            return User(response.json())
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def get_student_state(self, userId):
        '''Get user's student status
        Args:
            userId (str)  : user's unique identifier
        Returns:
            Migrated user object.
        Raises:
            TrivoreIDException if the status code is not 200.
        '''
        uri = su.uri(self._server, self._STUDENT_STATE).format(userId)
        response = self._session.get(uri, headers=self._header,
                                          auth=self._auth)

        if response.status_code is 200:
            logging.debug('Successfully got student state for user with id {}'
                                                                .format(userId))
            return StudentState(response.json())
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)

    def update_student_state(self, userId, state):
        '''Update user's student status
        Args:
            userId (str)  : user's unique identifier
        Returns:
            Updated user's status.
        Raises:
            TrivoreIDException if the status code is not 200.
            TrivoreIDSDKException if the type of the StudentState is wrong
        '''
        if 'StudentState' not in str(type(state)):
            raise TrivoreIDSDKException('StudentState type is wrong!')

        uri = su.uri(self._server, self._STUDENT_STATE).format(userId)
        response = self._session.post(uri, json=state.serialize(),
                                           headers=self._header,
                                           auth=self._auth)

        if response.status_code is 200:
            logging.debug('Successfully updated student state of the user {}'
                                                                .format(userId))
            return StudentState(response.json())
        else:
            raise TrivoreIDException(su.error_response_message(response),
                                     response.status_code)



import os
import sys

from setuptools import setup, find_packages

VERSION         = '2.0.10'
DESCRIPTION     = 'Wrapper for TrivoreID SDK'
AUTHOR          = 'Anastasia Gromova'
AUTHOR_EMAIL    = 'id-client-sdk@trivore.com'
LICENSE         = 'Apache 2.0'

this_directory = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    LONG_DESCRIPTION = f.read()

LONG_DESCRIPTION_CONTENT_TYPE = 'text/markdown'

def check_dependencies():
    install_requires = []

    try:
        import requests
    except ImportError:
        install_requires.append('requests')

    return install_requires

if __name__ == "__main__":
    install_requires = check_dependencies()

    setup(name='trivoreid',
        author=AUTHOR,
        author_email=AUTHOR_EMAIL,
        maintainer=AUTHOR,
        maintainer_email=AUTHOR_EMAIL,
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        long_description_content_type=LONG_DESCRIPTION_CONTENT_TYPE,
        license=LICENSE,
        version=VERSION,
        install_requires=install_requires,
        setup_requires=install_requires,
        packages=find_packages())
